@@ -1064,9 +1064,9 @@ layers_text_discard_cmd_callback (GimpAction *action,
 }
 
 void
-layers_text_to_path_cmd_callback (GimpAction *action,
-                                  GVariant   *value,
-                                  gpointer    data)
+layers_text_to_vectors_cmd_callback (GimpAction *action,
+                                     GVariant   *value,
+                                     gpointer    data)
 {
   GimpImage *image;
   GList     *layers;
@@ -1105,9 +1105,9 @@ layers_text_to_path_cmd_callback (GimpAction *action,
 }
 
 void
-layers_text_along_path_cmd_callback (GimpAction *action,
-                                     GVariant   *value,
-                                     gpointer    data)
+layers_text_along_vectors_cmd_callback (GimpAction *action,
+                                        GVariant   *value,
+                                        gpointer    data)
 {
   GimpImage   *image;
   GList       *layers;
