@@ -512,9 +512,6 @@ _("Enable the Seamless Clone tool.")
 #define PLAYGROUND_PAINT_SELECT_TOOL_BLURB \
 _("Enable the Paint Select tool.")
 
-#define PLAYGROUND_USE_LIST_BOX_BLURB \
-_("Use the new GtkListBox widget for simple lists.")
-
 #define SPACE_BAR_ACTION_BLURB \
 _("What to do when the space bar is pressed in the image window.")
 
@@ -667,22 +664,22 @@ _("Sets the default channel name for the 'New Channel' dialog.")
 #define CHANNEL_NEW_COLOR_BLURB \
 _("Sets the default color and opacity for the 'New Channel' dialog.")
 
-#define PATH_NEW_NAME_BLURB \
+#define VECTORS_NEW_NAME_BLURB \
 _("Sets the default path name for the 'New Path' dialog.")
 
-#define PATH_EXPORT_PATH_BLURB \
+#define VECTORS_EXPORT_PATH_BLURB \
 _("Sets the default folder path for the 'Export Path' dialog.")
 
-#define PATH_EXPORT_ACTIVE_ONLY_BLURB \
+#define VECTORS_EXPORT_ACTIVE_ONLY_BLURB \
 _("Sets the default 'Export the selected paths' state for the 'Export Path' dialog.")
 
-#define PATH_IMPORT_PATH_BLURB \
+#define VECTORS_IMPORT_PATH_BLURB \
 _("Sets the default folder path for the 'Import Path' dialog.")
 
-#define PATH_IMPORT_MERGE_BLURB \
+#define VECTORS_IMPORT_MERGE_BLURB \
 _("Sets the default 'Merge imported paths' state for the 'Import Path' dialog.")
 
-#define PATH_IMPORT_SCALE_BLURB \
+#define VECTORS_IMPORT_SCALE_BLURB \
 _("Sets the default 'Scale imported paths to fit size' state for the 'Import Path' dialog.")
 
 #define SELECTION_FEATHER_RADIUS_BLURB \
