@@ -26,6 +26,7 @@
 
 #include "core-types.h"
 
+#include "gimp.h"
 #include "gimpbezierdesc.h"
 #include "gimpbrush.h"
 #include "gimpbrush-boundary.h"
@@ -37,6 +38,7 @@
 #include "gimpbrushcache.h"
 #include "gimpbrushgenerated.h"
 #include "gimpbrushpipe.h"
+#include "gimpcontext.h"
 #include "gimptagged.h"
 #include "gimptempbuf.h"
 
@@ -77,7 +79,9 @@ static gboolean      gimp_brush_get_size              (GimpViewable         *vie
 static GimpTempBuf * gimp_brush_get_new_preview       (GimpViewable         *viewable,
                                                        GimpContext          *context,
                                                        gint                  width,
-                                                       gint                  height);
+                                                       gint                  height,
+                                                       GeglColor            *color,
+                                                       GeglColor            *background);
 static gchar       * gimp_brush_get_description       (GimpViewable         *viewable,
                                                        gchar               **tooltip);
 
@@ -270,7 +274,9 @@ static GimpTempBuf *
 gimp_brush_get_new_preview (GimpViewable *viewable,
                             GimpContext  *context,
                             gint          width,
-                            gint          height)
+                            gint          height,
+                            GeglColor    *color,
+                            GeglColor    *background)
 {
   GimpBrush         *brush       = GIMP_BRUSH (viewable);
   const GimpTempBuf *mask_buf    = brush->priv->mask;
@@ -366,13 +372,18 @@ gimp_brush_get_new_preview (GimpViewable *viewable,
     }
   else
     {
+      guint8 rgb[3] = {0, 0, 0};
+
+      if (color != NULL)
+        gegl_color_get_pixel (color, babl_format ("R'G'B' u8"), rgb);
+
       for (y = 0; y < mask_height; y++)
         {
           for (x = 0; x < mask_width ; x++)
             {
-              *buf++ = 0;
-              *buf++ = 0;
-              *buf++ = 0;
+              *buf++ = rgb[0];
+              *buf++ = rgb[1];
+              *buf++ = rgb[2];
               *buf++ = *mask++;
             }
         }
