@@ -87,7 +87,8 @@ static GimpTempBuf * gimp_curve_get_new_preview   (GimpViewable     *viewable,
                                                    GimpContext      *context,
                                                    gint              width,
                                                    gint              height,
-                                                   GeglColor        *fg_color);
+                                                   GeglColor        *color,
+                                                   GeglColor        *background);
 static gchar       * gimp_curve_get_description   (GimpViewable     *viewable,
                                                    gchar           **tooltip);
 
@@ -523,7 +524,8 @@ gimp_curve_get_new_preview (GimpViewable *viewable,
                             GimpContext  *context,
                             gint          width,
                             gint          height,
-                            GeglColor    *fg_color G_GNUC_UNUSED)
+                            GeglColor    *color G_GNUC_UNUSED,
+                            GeglColor    *background G_GNUC_UNUSED)
 {
   return NULL;
 }
