@@ -46,7 +46,6 @@
 #include "gimpdrawable-shadow.h"
 #include "gimpdrawable-transform.h"
 #include "gimpdrawablefilter.h"
-#include "gimpdrawablefiltermask.h"
 #include "gimpfilterstack.h"
 #include "gimpimage.h"
 #include "gimpimage-colormap.h"
@@ -112,9 +111,6 @@ static GeglNode * gimp_drawable_get_node           (GimpFilter        *filter);
 static void       gimp_drawable_removed            (GimpItem          *item);
 static GimpItem * gimp_drawable_duplicate          (GimpItem          *item,
                                                     GType              new_type);
-static void       gimp_drawable_convert            (GimpItem          *item,
-                                                    GimpImage         *dest_image,
-                                                    GType              old_type);
 static void       gimp_drawable_scale              (GimpItem          *item,
                                                     gint               new_width,
                                                     gint               new_height,
@@ -306,7 +302,6 @@ gimp_drawable_class_init (GimpDrawableClass *klass)
 
   item_class->removed             = gimp_drawable_removed;
   item_class->duplicate           = gimp_drawable_duplicate;
-  item_class->convert             = gimp_drawable_convert;
   item_class->scale               = gimp_drawable_scale;
   item_class->resize              = gimp_drawable_resize;
   item_class->flip                = gimp_drawable_flip;
@@ -588,7 +583,7 @@ gimp_drawable_duplicate (GimpItem *item,
 
               new_filter = gimp_drawable_filter_duplicate (new_drawable,
                                                            filter);
-              if (new_filter)
+              if (filter)
                 {
                   gimp_drawable_filter_apply (new_filter, NULL);
                   gimp_drawable_filter_commit (new_filter, TRUE, NULL, FALSE);
@@ -603,39 +598,6 @@ gimp_drawable_duplicate (GimpItem *item,
   return new_item;
 }
 
-static void
-gimp_drawable_convert (GimpItem  *item,
-                       GimpImage *dest_image,
-                       GType      old_type)
-{
-  GimpContainer *filters;
-
-  filters = gimp_drawable_get_filters (GIMP_DRAWABLE (item));
-  if (gimp_container_get_n_children (filters) > 0)
-    {
-      GList *filter_list;
-
-      for (filter_list = GIMP_LIST (filters)->queue->tail;
-           filter_list;
-           filter_list = g_list_previous (filter_list))
-        {
-          if (GIMP_IS_DRAWABLE_FILTER (filter_list->data))
-            {
-              GimpDrawableFilter     *filter = filter_list->data;
-              GimpDrawableFilterMask *mask;
-
-              mask = gimp_drawable_filter_get_mask (filter);
-
-              if (mask)
-                gimp_drawable_convert (GIMP_ITEM (mask), dest_image,
-                                       GIMP_TYPE_DRAWABLE_FILTER_MASK);
-            }
-        }
-    }
-
-  GIMP_ITEM_CLASS (parent_class)->convert (item, dest_image, old_type);
-}
-
 static void
 gimp_drawable_scale (GimpItem              *item,
                      gint                   new_width,
