@@ -173,7 +173,6 @@ static void       gimp_drawable_filter_lock_alpha_changed    (GimpLayer
 
 static void       gimp_drawable_filter_reorder               (GimpFilterStack    *stack,
                                                               GimpDrawableFilter *reordered_filter,
-                                                              gint                old_index,
                                                               gint                new_index,
                                                               GimpDrawableFilter *filter);
 
@@ -873,7 +872,7 @@ gimp_drawable_filter_update (GimpDrawableFilter      *filter,
                              GimpLayerColorSpace      composite_space,
                              GimpLayerCompositeMode   composite_mode,
                              const gchar            **auxinputnames,
-                             GimpDrawable           **auxinputs,
+                             const GimpDrawable     **auxinputs,
                              GError                 **error)
 {
   GimpImage    *image;
@@ -1332,13 +1331,12 @@ gimp_drawable_filter_layer_mask_freeze (GimpDrawableFilter *filter)
                                         filter);
 }
 
-void
-gimp_drawable_filter_refresh_crop (GimpDrawableFilter *filter,
-                                   GeglRectangle      *rect)
+void gimp_drawable_filter_refresh_crop (GimpDrawableFilter *filter,
+                                        GeglRectangle      *rect)
 {
   g_return_if_fail (GIMP_IS_DRAWABLE_FILTER (filter));
 
-  if (rect && filter->crop_enabled)
+  if (rect)
     {
       /* Some filters have built-in width/height properties that limit
        * the buffer size. If they have one in those roles, we can update
@@ -2017,7 +2015,6 @@ gimp_drawable_filter_lock_alpha_changed (GimpLayer          *layer,
 static void
 gimp_drawable_filter_reorder (GimpFilterStack    *stack,
                               GimpDrawableFilter *reordered_filter,
-                              gint                old_index,
                               gint                new_index,
                               GimpDrawableFilter *filter)
 {
@@ -2035,8 +2032,6 @@ gimp_drawable_filter_reorder (GimpFilterStack    *stack,
            * it's organized.
            */
           GIMP_IS_DRAWABLE_FILTER (GIMP_LIST (stack)->queue->head->data))
-        {
-          gimp_drawable_filter_sync_format (GIMP_LIST (stack)->queue->head->data);
-        }
+        gimp_drawable_filter_sync_format (GIMP_LIST (stack)->queue->head->data);
     }
 }
