@@ -60,7 +60,9 @@ static gboolean      gimp_gradient_get_popup_size    (GimpViewable        *viewa
 static GimpTempBuf * gimp_gradient_get_new_preview   (GimpViewable        *viewable,
                                                       GimpContext         *context,
                                                       gint                 width,
-                                                      gint                 height);
+                                                      gint                 height,
+                                                      GeglColor           *color,
+                                                      GeglColor           *background);
 
 static const gchar * gimp_gradient_get_extension     (GimpData            *data);
 static void          gimp_gradient_copy              (GimpData            *data,
@@ -213,7 +215,9 @@ static GimpTempBuf *
 gimp_gradient_get_new_preview (GimpViewable *viewable,
                                GimpContext  *context,
                                gint          width,
-                               gint          height)
+                               gint          height,
+                               GeglColor    *color G_GNUC_UNUSED,
+                               GeglColor    *background G_GNUC_UNUSED)
 {
   GimpGradient        *gradient = GIMP_GRADIENT (viewable);
   GimpGradientSegment *seg      = NULL;
