@@ -37,8 +37,6 @@
 #include "gimpchannel.h"
 #include "gimpcontext.h"
 #include "gimpdrawable-fill.h"
-#include "gimpdrawable-filters.h"
-#include "gimpdrawablefilter.h"
 #include "gimpgrouplayer.h"
 #include "gimpimage.h"
 #include "gimpimage-color-profile.h"
@@ -54,6 +52,11 @@
 #include "gimp-intl.h"
 
 
+static void   gimp_image_new_add_creation_metadata (GimpImage *image);
+
+
+/*  public functions  */
+
 GimpTemplate *
 gimp_image_new_get_last_template (Gimp      *gimp,
                                   GimpImage *image)
@@ -102,26 +105,6 @@ gimp_image_new_set_last_template (Gimp         *gimp,
                     G_OBJECT (gimp->image_new_last_template), 0);
 }
 
-void
-gimp_image_new_add_creation_metadata (GimpImage *image)
-{
-  GimpMetadata *metadata;
-
-  metadata = gimp_image_get_metadata (image);
-  if (! metadata)
-    {
-      g_critical ("Metadata not found. Should not happen!");
-    }
-  else
-    {
-      GDateTime *datetime;
-
-      datetime = g_date_time_new_now_local ();
-      gimp_metadata_set_creation_date (metadata, datetime);
-      g_date_time_unref (datetime);
-    }
-}
-
 GimpImage *
 gimp_image_new_from_template (Gimp         *gimp,
                               GimpTemplate *template,
@@ -225,7 +208,6 @@ gimp_image_new_from_drawable (Gimp         *gimp,
   gdouble            xres;
   gdouble            yres;
   GimpColorProfile  *profile;
-  GimpContainer     *filters;
 
   g_return_val_if_fail (GIMP_IS_GIMP (gimp), NULL);
   g_return_val_if_fail (GIMP_IS_DRAWABLE (drawable), NULL);
@@ -276,36 +258,6 @@ gimp_image_new_from_drawable (Gimp         *gimp,
   if (gimp_layer_can_lock_alpha (new_layer))
     gimp_layer_set_lock_alpha (new_layer, FALSE, FALSE);
 
-  filters = gimp_drawable_get_filters (GIMP_DRAWABLE (drawable));
-  if (gimp_container_get_n_children (filters) > 0)
-    {
-      GList *filter_list;
-
-      for (filter_list = GIMP_LIST (filters)->queue->tail;
-           filter_list;
-           filter_list = g_list_previous (filter_list))
-        {
-          if (GIMP_IS_DRAWABLE_FILTER (filter_list->data))
-            {
-              GimpDrawableFilter *old_filter = filter_list->data;
-              GimpDrawableFilter *filter;
-
-              filter =
-                gimp_drawable_filter_duplicate (GIMP_DRAWABLE (new_layer),
-                                                old_filter);
-
-              if (filter != NULL)
-                {
-                  gimp_drawable_filter_apply (filter, NULL);
-                  gimp_drawable_filter_commit (filter, TRUE, NULL, FALSE);
-
-                  gimp_drawable_filter_layer_mask_freeze (filter);
-                  g_object_unref (filter);
-                }
-            }
-        }
-    }
-
   gimp_image_add_layer (new_image, new_layer, NULL, 0, TRUE);
 
   gimp_image_new_add_creation_metadata (new_image);
@@ -397,11 +349,10 @@ gimp_image_new_copy_drawables (GimpImage *image,
     {
       if (g_list_find (copied_drawables, iter->data))
         {
-          GimpLayer     *new_layer;
-          GimpContainer *filters;
-          GType          new_type;
-          gboolean       is_group;
-          gboolean       is_tagged;
+          GimpLayer *new_layer;
+          GType      new_type;
+          gboolean   is_group;
+          gboolean   is_tagged;
 
           if (GIMP_IS_LAYER (iter->data))
             new_type = G_TYPE_FROM_INSTANCE (iter->data);
@@ -445,33 +396,6 @@ gimp_image_new_copy_drawables (GimpImage *image,
           if (gimp_layer_can_lock_alpha (new_layer))
             gimp_layer_set_lock_alpha (new_layer, FALSE, FALSE);
 
-          filters = gimp_drawable_get_filters (GIMP_DRAWABLE (iter->data));
-          if (gimp_container_get_n_children (filters) > 0)
-            {
-              GList *filter_list;
-
-              for (filter_list = GIMP_LIST (filters)->queue->tail; filter_list;
-                   filter_list = g_list_previous (filter_list))
-                {
-                  if (GIMP_IS_DRAWABLE_FILTER (filter_list->data))
-                    {
-                      GimpDrawableFilter *old_filter = filter_list->data;
-                      GimpDrawableFilter *filter;
-
-                      filter = gimp_drawable_filter_duplicate (GIMP_DRAWABLE (new_layer), old_filter);
-
-                      if (filter != NULL)
-                        {
-                          gimp_drawable_filter_apply (filter, NULL);
-                          gimp_drawable_filter_commit (filter, TRUE, NULL, FALSE);
-
-                          gimp_drawable_filter_layer_mask_freeze (filter);
-                          g_object_unref (filter);
-                        }
-                    }
-                }
-            }
-
           gimp_image_add_layer (new_image, new_layer, new_parent, index++, TRUE);
 
           /* If a group, loop through children. */
@@ -728,3 +652,26 @@ gimp_image_new_from_pixbuf (Gimp        *gimp,
 
   return new_image;
 }
+
+
+/*  private functions  */
+
+static void
+gimp_image_new_add_creation_metadata (GimpImage *image)
+{
+  GimpMetadata *metadata;
+
+  metadata = gimp_image_get_metadata (image);
+  if (! metadata)
+    {
+      g_critical ("Metadata not found. Should not happen!");
+    }
+  else
+    {
+      GDateTime *datetime;
+
+      datetime = g_date_time_new_now_local ();
+      gimp_metadata_set_creation_date (metadata, datetime);
+      g_date_time_unref (datetime);
+    }
+}
