@@ -121,7 +121,9 @@ GimpTempBuf *
 gimp_image_get_new_preview (GimpViewable *viewable,
                             GimpContext  *context,
                             gint          width,
-                            gint          height)
+                            gint          height,
+                            GeglColor    *color G_GNUC_UNUSED,
+                            GeglColor    *background G_GNUC_UNUSED)
 {
   GimpImage   *image = GIMP_IMAGE (viewable);
   const Babl  *format;
@@ -150,7 +152,9 @@ GdkPixbuf *
 gimp_image_get_new_pixbuf (GimpViewable *viewable,
                            GimpContext  *context,
                            gint          width,
-                           gint          height)
+                           gint          height,
+                           GeglColor    *color G_GNUC_UNUSED,
+                           GeglColor    *background G_GNUC_UNUSED)
 {
   GimpImage          *image = GIMP_IMAGE (viewable);
   GdkPixbuf          *pixbuf;
