@@ -94,12 +94,14 @@ static GimpTempBuf      * gimp_image_proxy_get_new_preview          (GimpViewabl
                                                                      GimpContext                *context,
                                                                      gint                        width,
                                                                      gint                        height,
-                                                                     GeglColor                  *fg_color);
+                                                                     GeglColor                  *color,
+                                                                     GeglColor                  *background);
 static GdkPixbuf        * gimp_image_proxy_get_new_pixbuf           (GimpViewable               *viewable,
                                                                      GimpContext                *context,
                                                                      gint                        width,
                                                                      gint                        height,
-                                                                     GeglColor                  *fg_color);
+                                                                     GeglColor                  *color,
+                                                                     GeglColor                  *background);
 static gchar            * gimp_image_proxy_get_description          (GimpViewable               *viewable,
                                                                      gchar                     **tooltip);
 
@@ -376,7 +378,8 @@ gimp_image_proxy_get_new_preview (GimpViewable *viewable,
                                   GimpContext  *context,
                                   gint          width,
                                   gint          height,
-                                  GeglColor    *fg_color G_GNUC_UNUSED)
+                                  GeglColor    *color G_GNUC_UNUSED,
+                                  GeglColor    *background G_GNUC_UNUSED)
 {
   GimpImageProxy *image_proxy = GIMP_IMAGE_PROXY (viewable);
   GimpImage      *image       = image_proxy->priv->image;
@@ -418,7 +421,8 @@ gimp_image_proxy_get_new_pixbuf (GimpViewable *viewable,
                                  GimpContext  *context,
                                  gint          width,
                                  gint          height,
-                                 GeglColor    *fg_color G_GNUC_UNUSED)
+                                 GeglColor    *color G_GNUC_UNUSED,
+                                 GeglColor    *background G_GNUC_UNUSED)
 {
   GimpImageProxy     *image_proxy = GIMP_IMAGE_PROXY (viewable);
   GimpImage          *image       = image_proxy->priv->image;
