@@ -76,7 +76,8 @@ static GimpTempBuf * gimp_palette_get_new_preview     (GimpViewable         *vie
                                                        GimpContext          *context,
                                                        gint                  width,
                                                        gint                  height,
-                                                       GeglColor            *fg_color);
+                                                       GeglColor            *color,
+                                                       GeglColor            *background);
 static gchar       * gimp_palette_get_description     (GimpViewable         *viewable,
                                                        gchar               **tooltip);
 static const gchar * gimp_palette_get_extension       (GimpData             *data);
@@ -126,7 +127,6 @@ gimp_palette_class_init (GimpPaletteClass *klass)
   gimp_object_class->get_memsize    = gimp_palette_get_memsize;
 
   viewable_class->default_icon_name = "gtk-select-color";
-  viewable_class->default_name      = _("Palette");
   viewable_class->get_preview_size  = gimp_palette_get_preview_size;
   viewable_class->get_popup_size    = gimp_palette_get_popup_size;
   viewable_class->get_new_preview   = gimp_palette_get_new_preview;
@@ -242,7 +242,8 @@ gimp_palette_get_new_preview (GimpViewable *viewable,
                               GimpContext  *context,
                               gint          width,
                               gint          height,
-                              GeglColor    *fg_color G_GNUC_UNUSED)
+                              GeglColor    *color G_GNUC_UNUSED,
+                              GeglColor    *background G_GNUC_UNUSED)
 {
   GimpPalette *palette  = GIMP_PALETTE (viewable);
   GimpTempBuf *temp_buf;
