@@ -62,10 +62,10 @@ struct _ResizeDialog
 
   gdouble             old_xres;
   gdouble             old_yres;
-  GimpUnit            old_res_unit;
+  GimpUnit           *old_res_unit;
   gint                old_width;
   gint                old_height;
-  GimpUnit            old_unit;
+  GimpUnit           *old_unit;
   GimpFillType        old_fill_type;
   GimpItemSet         old_layer_set;
   gboolean            old_resize_text_layers;
@@ -123,7 +123,7 @@ resize_dialog_new (GimpViewable       *viewable,
                    GtkWidget          *parent,
                    GimpHelpFunc        help_func,
                    const gchar        *help_id,
-                   GimpUnit            unit,
+                   GimpUnit           *unit,
                    GimpFillType        fill_type,
                    GimpItemSet         layer_set,
                    gboolean            resize_text_layers,
@@ -388,7 +388,7 @@ resize_dialog_new (GimpViewable       *viewable,
   gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton), TRUE);
   gtk_entry_set_width_chars (GTK_ENTRY (spinbutton), SB_WIDTH);
 
-  private->offset = entry = gimp_size_entry_new (1, unit, "%p",
+  private->offset = entry = gimp_size_entry_new (1, unit, "%n",
                                                  TRUE, FALSE, FALSE, SB_WIDTH,
                                                  GIMP_SIZE_ENTRY_UPDATE_SIZE);
   gimp_size_entry_add_field (GIMP_SIZE_ENTRY (entry),
@@ -419,6 +419,8 @@ resize_dialog_new (GimpViewable       *viewable,
   gtk_widget_set_halign (frame, GTK_ALIGN_CENTER);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
   gtk_box_pack_start (GTK_BOX (center_right_vbox), frame, FALSE, FALSE, 0);
+  gtk_style_context_add_class (gtk_widget_get_style_context (frame),
+                               "gimp-offset-area-frame");
   gtk_widget_show (frame);
 
   private->area = gimp_offset_area_new (width, height);
@@ -427,7 +429,7 @@ resize_dialog_new (GimpViewable       *viewable,
 
   gimp_viewable_get_preview_size (viewable, 200, TRUE, TRUE, &width, &height);
   pixbuf = gimp_viewable_get_pixbuf (viewable, context,
-                                     width, height);
+                                     width, height, NULL, NULL);
 
   if (pixbuf)
     gimp_offset_area_set_pixbuf (GIMP_OFFSET_AREA (private->area), pixbuf);
@@ -552,12 +554,12 @@ resize_dialog_response (GtkWidget    *dialog,
                         ResizeDialog *private)
 {
   GimpSizeEntry *entry = GIMP_SIZE_ENTRY (private->offset);
-  GimpUnit       unit;
+  GimpUnit      *unit;
   gint           width;
   gint           height;
   gdouble        xres;
   gdouble        yres;
-  GimpUnit       res_unit;
+  GimpUnit      *res_unit;
 
   switch (response_id)
     {
@@ -606,13 +608,20 @@ resize_dialog_reset (ResizeDialog *private)
                 NULL);
 
   g_object_set (private->box,
-                "width",           private->old_width,
-                "height",          private->old_height,
                 "unit",            private->old_unit,
                 "xresolution",     private->old_xres,
                 "yresolution",     private->old_yres,
                 "resolution-unit", private->old_res_unit,
                 NULL);
+  /**
+   * reset width and height after the other properties to avoid the problems
+   * noted in issue #10225
+   **/
+
+  g_object_set (private->box,
+                "width",           private->old_width,
+                "height",          private->old_height,
+                NULL);
 
   if (private->layer_set_combo)
     gimp_int_combo_box_set_active (GIMP_INT_COMBO_BOX (private->layer_set_combo),
@@ -720,7 +729,7 @@ template_changed (GimpContext  *context,
                   GimpTemplate *template,
                   ResizeDialog *private)
 {
-  GimpUnit unit = private->old_unit;
+  GimpUnit *unit = private->old_unit;
 
   private->template = template;
 
@@ -729,10 +738,10 @@ template_changed (GimpContext  *context,
 
   if (template != NULL)
     {
-      gdouble  xres;
-      gdouble  yres;
-      GimpUnit res_unit;
-      gboolean resolution_mismatch;
+      gdouble   xres;
+      gdouble   yres;
+      GimpUnit *res_unit;
+      gboolean  resolution_mismatch;
 
       unit     = gimp_template_get_unit            (template);
       xres     = gimp_template_get_resolution_x    (template);
@@ -743,8 +752,7 @@ template_changed (GimpContext  *context,
                             yres     != private->old_yres ||
                             res_unit != private->old_res_unit;
 
-      if (resolution_mismatch &&
-          unit != GIMP_UNIT_PIXEL)
+      if (resolution_mismatch && unit != gimp_unit_pixel ())
         {
           gchar *text;
 
@@ -776,10 +784,10 @@ ppi_select_toggled (GtkWidget    *radio,
 {
   gint             width;
   gint             height;
-  GimpUnit         unit;
+  GimpUnit        *unit;
   gdouble          xres;
   gdouble          yres;
-  GimpUnit         res_unit;
+  GimpUnit        *res_unit;
   GtkToggleButton *image_button;
   gboolean         use_image_ppi;
 
@@ -803,8 +811,7 @@ ppi_select_toggled (GtkWidget    *radio,
       res_unit = gimp_template_get_resolution_unit (private->template);
     }
 
-  if (private->template != NULL &&
-      unit != GIMP_UNIT_PIXEL)
+  if (private->template != NULL && unit != gimp_unit_pixel ())
     {
       if (use_image_ppi)
         {
