@@ -55,7 +55,6 @@
 #include "widgets/gimpaction-history.h"
 #include "widgets/gimpclipboard.h"
 #include "widgets/gimpcolorselectorpalette.h"
-#include "widgets/gimpcontrollermanager.h"
 #include "widgets/gimpcontrollers.h"
 #include "widgets/gimpdevices.h"
 #include "widgets/gimpdialogfactory.h"
@@ -89,9 +88,9 @@
 #include "themes.h"
 
 #ifdef G_OS_WIN32
-#include <windows.h>
 #include <windef.h>
 #include <winbase.h>
+#include <windows.h>
 #endif
 
 #ifdef GDK_WINDOWING_QUARTZ
@@ -548,12 +547,11 @@ static void
 gui_restore_after_callback (Gimp               *gimp,
                             GimpInitStatusFunc  status_callback)
 {
-  GimpGuiConfig         *gui_config = GIMP_GUI_CONFIG (gimp->config);
-  GimpControllerManager *controller_manager;
-  GimpUIManager         *image_ui_manager;
-  GimpDisplay           *display;
+  GimpGuiConfig *gui_config = GIMP_GUI_CONFIG (gimp->config);
+  GimpUIManager *image_ui_manager;
+  GimpDisplay   *display;
 #ifdef G_OS_WIN32
-  STARTUPINFO            StartupInfo;
+  STARTUPINFO    StartupInfo;
 
   GetStartupInfo (&StartupInfo);
 #endif
@@ -595,8 +593,7 @@ gui_restore_after_callback (Gimp               *gimp,
                     gimp);
 
   gimp_devices_restore (gimp);
-  controller_manager = gimp_get_controller_manager (gimp);
-  gimp_controller_manager_restore (controller_manager, image_ui_manager);
+  gimp_controllers_restore (gimp, image_ui_manager);
   modifiers_restore (gimp);
 
   if (status_callback == splash_update)
@@ -692,12 +689,7 @@ gui_exit_callback (Gimp     *gimp,
     gimp_devices_save (gimp, FALSE);
 
   if (TRUE /* gui_config->save_controllers */)
-    {
-      GimpControllerManager *controller_manager;
-
-      controller_manager = gimp_get_controller_manager (gimp);
-      gimp_controller_manager_save (controller_manager);
-    }
+    gimp_controllers_save (gimp);
 
   modifiers_save (gimp, FALSE);
 
