@@ -22,6 +22,10 @@
 #include <gegl.h>
 #include <gtk/gtk.h>
 
+#ifdef G_OS_WIN32
+#include <windows.h>
+#endif
+
 #include "libgimpbase/gimpbase.h"
 #include "libgimpconfig/gimpconfig.h"
 
@@ -43,25 +47,36 @@
 
 /*  local function prototypes  */
 
-static void   themes_apply_theme         (Gimp                   *gimp,
-                                          GimpGuiConfig          *config);
-static void   themes_list_themes_foreach (gpointer                key,
-                                          gpointer                value,
-                                          gpointer                data);
-static gint   themes_name_compare        (const void             *p1,
-                                          const void             *p2);
-static void   themes_theme_change_notify (GimpGuiConfig          *config,
-                                          GParamSpec             *pspec,
-                                          Gimp                   *gimp);
-static void   themes_theme_paths_notify  (GimpExtensionManager   *manager,
-                                          GParamSpec             *pspec,
-                                          Gimp                   *gimp);
+static void   themes_apply_theme                   (Gimp                  *gimp,
+                                                    GimpGuiConfig         *config);
+static void   themes_list_themes_foreach           (gpointer               key,
+                                                    gpointer               value,
+                                                    gpointer               data);
+static gint   themes_name_compare                  (const void            *p1,
+                                                    const void            *p2);
+static void   themes_theme_paths_notify            (GimpExtensionManager  *manager,
+                                                    GParamSpec            *pspec,
+                                                    Gimp                  *gimp);
+#if defined(G_OS_UNIX) && ! defined(__APPLE__)
+static void   themes_theme_settings_portal_changed (GDBusProxy            *proxy,
+                                                    const gchar           *sender_name,
+                                                    const gchar           *signal_name,
+                                                    GVariant              *parameters,
+                                                    Gimp                  *gimp);
+#endif
+
+#ifdef G_OS_WIN32
+static gboolean themes_win32_is_darkmode_active    (void);
+#endif
 
 
 /*  private variables  */
 
-static GHashTable       *themes_hash           = NULL;
-static GtkStyleProvider *themes_style_provider = NULL;
+static GHashTable       *themes_hash            = NULL;
+static GtkStyleProvider *themes_style_provider  = NULL;
+#if defined(G_OS_UNIX) && ! defined(__APPLE__)
+static GDBusProxy       *themes_settings_portal = NULL;
+#endif
 
 
 /*  public functions  */
@@ -70,6 +85,9 @@ void
 themes_init (Gimp *gimp)
 {
   GimpGuiConfig *config;
+#if defined(G_OS_UNIX) && ! defined(__APPLE__)
+  GError        *error = NULL;
+#endif
 
   g_return_if_fail (GIMP_IS_GIMP (gimp));
 
@@ -90,6 +108,29 @@ themes_init (Gimp *gimp)
                                              themes_style_provider,
                                              GTK_STYLE_PROVIDER_PRIORITY_APPLICATION + 1);
 
+#if defined(G_OS_UNIX) && ! defined(__APPLE__)
+  themes_settings_portal =
+    g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
+                                   G_DBUS_PROXY_FLAGS_NONE, NULL,
+                                   "org.freedesktop.portal.Desktop",
+                                   "/org/freedesktop/portal/desktop",
+                                   "org.freedesktop.portal.Settings",
+                                   NULL, &error);
+
+  if (error)
+    {
+      g_printerr ("Could not access portal: %s", error->message);
+      g_clear_error (&error);
+    }
+  else
+    {
+      g_signal_connect (themes_settings_portal,
+                        "g-signal::SettingChanged",
+                        G_CALLBACK (themes_theme_settings_portal_changed),
+                        gimp);
+    }
+#endif
+
   g_signal_connect (config, "notify::theme",
                     G_CALLBACK (themes_theme_change_notify),
                     gimp);
@@ -135,6 +176,10 @@ themes_exit (Gimp *gimp)
     }
 
   g_clear_object (&themes_style_provider);
+
+#if defined(G_OS_UNIX) && ! defined(__APPLE__)
+  g_clear_object (&themes_settings_portal);
+#endif
 }
 
 gchar **
@@ -231,6 +276,44 @@ themes_get_theme_file (Gimp        *gimp,
   return file;
 }
 
+void
+themes_theme_change_notify (GimpGuiConfig *config,
+                            GParamSpec    *pspec,
+                            Gimp          *gimp)
+{
+  GFile  *theme_css;
+  GError *error = NULL;
+
+  g_object_set (gtk_settings_get_for_screen (gdk_screen_get_default ()),
+                "gtk-application-prefer-dark-theme",
+                config->theme_scheme != GIMP_THEME_LIGHT,
+                NULL);
+
+  themes_apply_theme (gimp, config);
+
+  theme_css = gimp_directory_file ("theme.css", NULL);
+
+  if (gimp->be_verbose)
+    g_print ("Parsing '%s'\n",
+             gimp_file_get_utf8_name (theme_css));
+
+  if (! gtk_css_provider_load_from_file (GTK_CSS_PROVIDER (themes_style_provider),
+                                         theme_css, &error))
+    {
+      g_printerr ("%s: error parsing %s: %s\n", G_STRFUNC,
+                  gimp_file_get_utf8_name (theme_css), error->message);
+      g_clear_error (&error);
+    }
+
+  g_object_unref (theme_css);
+
+  gtk_style_context_reset_widgets (gdk_screen_get_default ());
+
+#ifdef G_OS_WIN32
+  themes_set_title_bar (gimp);
+#endif
+}
+
 
 /*  private functions  */
 
@@ -238,15 +321,106 @@ static void
 themes_apply_theme (Gimp          *gimp,
                     GimpGuiConfig *config)
 {
-  GFile         *theme_css;
-  GOutputStream *output;
-  GError        *error = NULL;
-  gboolean       prefer_dark_theme;
+  GFile           *theme_css;
+  GOutputStream   *output;
+  GError          *error = NULL;
+  gboolean         prefer_dark_theme;
+  GimpThemeScheme  color_scheme;
 
   g_return_if_fail (GIMP_IS_GIMP (gimp));
   g_return_if_fail (GIMP_IS_GUI_CONFIG (config));
 
-  prefer_dark_theme = (config->theme_scheme != GIMP_THEME_LIGHT);
+#if defined(G_OS_UNIX) && ! defined(__APPLE__)
+  if (themes_settings_portal && config->theme_scheme == GIMP_THEME_SYSTEM)
+    {
+      GVariant *tuple_variant;
+      GVariant *variant;
+
+      tuple_variant = g_dbus_proxy_call_sync (themes_settings_portal,
+                                              "ReadOne",
+                                              g_variant_new ("(ss)", "org.freedesktop.appearance", "color-scheme"),
+                                              G_DBUS_CALL_FLAGS_NONE,
+                                              G_MAXINT,
+                                              NULL,
+                                              &error);
+      if (error)
+        {
+          g_clear_error (&error);
+
+          tuple_variant = g_dbus_proxy_call_sync (themes_settings_portal,
+                                                  "Read",
+                                                  g_variant_new ("(ss)", "org.freedesktop.appearance", "color-scheme"),
+                                                  G_DBUS_CALL_FLAGS_NONE,
+                                                  G_MAXINT,
+                                                  NULL,
+                                                  &error);
+          if (!error)
+            {
+              /*
+               * Since the org.freedesktop.portal.Settings.Read method returns
+               * two layers of variant, re-assign the tuple.
+               * https://flatpak.github.io/xdg-desktop-portal/docs/doc-org.freedesktop.portal.Settings.html#org-freedesktop-portal-settings-read
+               */
+              g_variant_get (tuple_variant, "(v)", &tuple_variant);
+
+              g_variant_get (tuple_variant, "v", &variant);
+            }
+        }
+      else
+        {
+          g_variant_get (tuple_variant, "(v)", &variant);
+        }
+
+      if (!error)
+        {
+          /*
+           * 1 means "Prefer dark", see:
+           * https://flatpak.github.io/xdg-desktop-portal/docs/doc-org.freedesktop.portal.Settings.html#description
+           */
+          prefer_dark_theme = (g_variant_get_uint32 (variant) == 1);
+
+          /*
+           * Note that normally it's a tri-state flag, with a
+           * prefer-light and no-preference case too, except that it
+           * looks like both in KDE and GNOME at least, they only set
+           * prefer-dark or no-preference and for us, the latter should
+           * also mean dark.
+           * Therefore for this setting to actually mean something, we
+           * are currently breaking the spec by having no-preference
+           * mean prefer-light. This should be fixed if/when the main
+           * desktops actually implement all 3 options some day.
+           */
+          color_scheme = prefer_dark_theme ? GIMP_THEME_DARK : GIMP_THEME_LIGHT;
+        }
+      else
+        {
+          g_printerr ("%s\n", error->message);
+          g_clear_error (&error);
+
+          color_scheme = (config->theme_scheme == GIMP_THEME_SYSTEM) ? GIMP_THEME_DARK : config->theme_scheme;
+          prefer_dark_theme = (color_scheme == GIMP_THEME_DARK ||
+                               color_scheme == GIMP_THEME_GRAY);
+        }
+    }
+  else
+#elif defined(G_OS_WIN32)
+  if (config->theme_scheme == GIMP_THEME_SYSTEM)
+    {
+      prefer_dark_theme = themes_win32_is_darkmode_active ();
+      color_scheme = prefer_dark_theme ? GIMP_THEME_DARK : GIMP_THEME_LIGHT;
+    }
+  else
+#endif
+    {
+      color_scheme = (config->theme_scheme == GIMP_THEME_SYSTEM) ? GIMP_THEME_DARK : config->theme_scheme;
+      prefer_dark_theme = (color_scheme == GIMP_THEME_DARK ||
+                           color_scheme == GIMP_THEME_GRAY);
+    }
+
+  g_object_set (gtk_settings_get_for_screen (gdk_screen_get_default ()),
+                "gtk-application-prefer-dark-theme", prefer_dark_theme,
+                NULL);
+
   theme_css = gimp_directory_file ("theme.css", NULL);
 
   if (gimp->be_verbose)
@@ -291,7 +465,7 @@ themes_apply_theme (Gimp          *gimp,
           if (! g_file_query_exists (dark, NULL))
             g_clear_object (&dark);
 
-          switch (config->theme_scheme)
+          switch (color_scheme)
             {
             case GIMP_THEME_LIGHT:
               if (light != NULL)
@@ -323,6 +497,8 @@ themes_apply_theme (Gimp          *gimp,
               else if (light != NULL)
                 file = g_object_ref (light);
               break;
+            case GIMP_THEME_SYSTEM:
+              g_return_if_reached ();
             }
 
           if (file != NULL)
@@ -353,7 +529,7 @@ themes_apply_theme (Gimp          *gimp,
           css_files = g_slist_prepend (css_files, g_file_new_for_path (tmp));
           g_free (tmp);
 
-          switch (config->theme_scheme)
+          switch (color_scheme)
             {
             case GIMP_THEME_LIGHT:
               tmp = g_build_filename (gimp_data_directory (),
@@ -370,6 +546,8 @@ themes_apply_theme (Gimp          *gimp,
                                       "themes", "Default", "gimp-dark.css",
                                       NULL);
               break;
+            case GIMP_THEME_SYSTEM:
+              g_return_if_reached ();
             }
 
           css_files = g_slist_prepend (css_files, g_file_new_for_path (tmp));
@@ -480,8 +658,6 @@ themes_apply_theme (Gimp          *gimp,
             "\n"
             "* { -GimpDock-tool-icon-size: %s; }"
             "\n"
-            "* { -GimpViewableDialog-tool-icon-size: %s; }"
-            "\n"
             "* { -GimpDockbook-tab-icon-size: %s; }"
             "\n"
             "* { -GimpColorNotebook-tab-icon-size: %s; }"
@@ -499,9 +675,9 @@ themes_apply_theme (Gimp          *gimp,
             "button, tab { padding: %dpx; }"
             "\n"
             "paned separator { padding: %dpx; }",
-            tool_icon_size, tool_icon_size, tool_icon_size, tab_icon_size,
-            tab_icon_size, tab_icon_size, button_icon_size, button_icon_size,
-            tool_icon_size, pal_padding, tab_padding, sep_padding);
+            tool_icon_size, tool_icon_size, tab_icon_size, tab_icon_size,
+            tab_icon_size, button_icon_size, button_icon_size, tool_icon_size,
+            pal_padding, tab_padding, sep_padding);
         }
 
       if (! error && config->font_relative_size != 1.0)
@@ -575,43 +751,6 @@ themes_name_compare (const void *p1,
   return strcmp (* (char **) p1, * (char **) p2);
 }
 
-static void
-themes_theme_change_notify (GimpGuiConfig *config,
-                            GParamSpec    *pspec,
-                            Gimp          *gimp)
-{
-  GFile  *theme_css;
-  GError *error = NULL;
-
-  g_object_set (gtk_settings_get_for_screen (gdk_screen_get_default ()),
-                "gtk-application-prefer-dark-theme", config->theme_scheme != GIMP_THEME_LIGHT,
-                NULL);
-
-  themes_apply_theme (gimp, config);
-
-  theme_css = gimp_directory_file ("theme.css", NULL);
-
-  if (gimp->be_verbose)
-    g_print ("Parsing '%s'\n",
-             gimp_file_get_utf8_name (theme_css));
-
-  if (! gtk_css_provider_load_from_file (GTK_CSS_PROVIDER (themes_style_provider),
-                                         theme_css, &error))
-    {
-      g_printerr ("%s: error parsing %s: %s\n", G_STRFUNC,
-                  gimp_file_get_utf8_name (theme_css), error->message);
-      g_clear_error (&error);
-    }
-
-  g_object_unref (theme_css);
-
-  gtk_style_context_reset_widgets (gdk_screen_get_default ());
-
-#ifdef G_OS_WIN32
-  themes_set_title_bar (gimp);
-#endif
-}
-
 static void
 themes_theme_paths_notify (GimpExtensionManager *manager,
                            GParamSpec           *pspec,
@@ -691,6 +830,33 @@ themes_theme_paths_notify (GimpExtensionManager *manager,
     }
 }
 
+#if defined(G_OS_UNIX) && ! defined(__APPLE__)
+static void
+themes_theme_settings_portal_changed (GDBusProxy  *proxy,
+                                      const gchar *sender_name,
+                                      const gchar *signal_name,
+                                      GVariant    *parameters,
+                                      Gimp        *gimp)
+{
+  const char *namespace;
+  const char *name;
+  GVariant   *value = NULL;
+
+  if (g_strcmp0 (signal_name, "SettingChanged"))
+    return;
+
+  g_variant_get (parameters, "(&s&sv)", &namespace, &name, &value);
+
+  if (g_strcmp0 (namespace, "org.freedesktop.appearance") == 0 &&
+      g_strcmp0 (name, "color-scheme") == 0)
+    {
+      themes_theme_change_notify (GIMP_GUI_CONFIG (gimp->config), NULL, gimp);
+    }
+
+  g_variant_unref (value);
+}
+#endif
+
 #ifdef G_OS_WIN32
 void
 themes_set_title_bar (Gimp *gimp)
@@ -708,4 +874,22 @@ themes_set_title_bar (Gimp *gimp)
   if (windows)
     g_list_free (windows);
 }
+
+static gboolean
+themes_win32_is_darkmode_active (void)
+{
+  DWORD   val      = 0;
+  DWORD   val_size = sizeof (val);
+  LSTATUS status;
+
+  status = RegGetValueA(HKEY_CURRENT_USER,
+                        "Software\\Microsoft\\Windows\\CurrentVersion\\Themes\\Personalize",
+                        "AppsUseLightTheme",
+                        RRF_RT_REG_DWORD,
+                        NULL,
+                        &val,
+                        &val_size);
+
+  return status == ERROR_SUCCESS && val == 0;
+}
 #endif
