@@ -18,19 +18,23 @@
 #ifndef __THEMES_H__
 #define __THEMES_H__
 
+void     themes_init                 (Gimp          *gimp);
+void     themes_exit                 (Gimp          *gimp);
 
-void     themes_init           (Gimp        *gimp);
-void     themes_exit           (Gimp        *gimp);
+gchar ** themes_list_themes          (Gimp          *gimp,
+                                      gint          *n_themes);
+GFile  * themes_get_theme_dir        (Gimp          *gimp,
+                                      const gchar   *theme_name);
+GFile  * themes_get_theme_file       (Gimp          *gimp,
+                                      const gchar   *first_component,
+                                      ...) G_GNUC_NULL_TERMINATED;
+
+void     themes_theme_change_notify  (GimpGuiConfig *config,
+                                      GParamSpec    *pspec,
+                                      Gimp          *gimp);
 
-gchar ** themes_list_themes    (Gimp        *gimp,
-                                gint        *n_themes);
-GFile  * themes_get_theme_dir  (Gimp        *gimp,
-                                const gchar *theme_name);
-GFile  * themes_get_theme_file (Gimp        *gimp,
-                                const gchar *first_component,
-                                ...) G_GNUC_NULL_TERMINATED;
 #ifdef G_OS_WIN32
-void     themes_set_title_bar  (Gimp        *gimp);
+void     themes_set_title_bar        (Gimp          *gimp);
 #endif
 
 #endif /* __THEMES_H__ */
