@@ -1,4 +1,4 @@
-﻿/* GIMP - The GNU Image Manipulation Program
+/* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
  * This program is free software: you can redistribute it and/or modify
@@ -116,9 +116,9 @@ static void      gimp_init_i18n               (void);
 static void      gimp_init_malloc             (void);
 
 #if defined (G_OS_WIN32) && !defined (GIMP_CONSOLE_COMPILATION)
-static void      gimp_attach_console_window     (void);
+static void      gimp_open_console_window     (void);
 #else
-#define gimp_attach_console_window() /* as nothing */
+#define gimp_open_console_window() /* as nothing */
 #endif
 
 static const gchar        *system_gimprc     = NULL;
@@ -184,20 +184,7 @@ static const GOptionEntry main_entries[] =
     N_("Open images as new"), NULL
   },
   {
-    "no-interface", 'i',
-#ifdef GIMP_CONSOLE_COMPILATION
-    /* The CLI option is not removed for gimp-console because it allows
-     * to set the same options on either the GUI or the console
-     * binaries, as a no-brainer. We in fact also use this in build
-     * scripts which may make use of either one of the binaries to
-     * construct some images.
-     * Yet it's unnecessary to show this option in the CLI tool usage as
-     * it's basically a no-op.
-     */
-    G_OPTION_FLAG_HIDDEN,
-#else
-    0,
-#endif
+    "no-interface", 'i', 0,
     G_OPTION_ARG_NONE, &no_interface,
     N_("Run without a user interface"), NULL
   },
@@ -310,7 +297,7 @@ static const GOptionEntry main_entries[] =
     N_("Show a preferences page with experimental features"), NULL
   },
   {
-    "show-debug-menu", 0, 0,
+    "show-debug-menu", 0, G_OPTION_FLAG_HIDDEN,
     G_OPTION_ARG_NONE, &show_debug_menu,
     N_("Show an image submenu with debug actions"), NULL
   },
@@ -327,10 +314,8 @@ static void
 gimp_macos_setenv (const char * progname)
 {
   /* helper to set environment variables for GIMP to be relocatable.
-   * Due to the changes on macOS 10.15, it is not recommended to set it in
-   * a shell wrapper inside Contents/MacOS (like AppImage's AppRun) anymore.
-   * LSEnvironment on Info.plist, limited to bundle scope, would not be enough.
-   * That way, we make sure our python is called instead of system one etc
+   * Due to the latest changes it is not recommended to set it in the shell
+   * wrapper anymore.
    */
   gchar  *resolved_path;
   /* on some OSX installations open file limit is 256 and GIMP needs more */
@@ -342,66 +327,59 @@ gimp_macos_setenv (const char * progname)
   resolved_path = g_canonicalize_filename (progname, NULL);
   if (resolved_path && ! g_getenv ("GIMP_NO_WRAPPER"))
     {
+      /* set path to the app folder to make sure that our python is called
+       * instead of system one
+       */
       static gboolean            show_playground   = TRUE;
 
       gchar   *path;
       gchar   *tmp;
-      gchar   *bin_dir;
-      gchar   *lib_dir;
-      gchar   *share_dir;
-      gchar   *etc_dir;
+      gchar   *app_dir;
+      gchar   *res_dir;
       size_t   path_len;
       struct   stat sb;
       gboolean need_pythonhome = TRUE;
 
-      bin_dir = g_path_get_dirname (resolved_path);
-      tmp = g_strdup_printf ("%s/../Resources/lib", bin_dir);
-      lib_dir = g_canonicalize_filename (tmp, NULL);
+      app_dir = g_path_get_dirname (resolved_path);
+      tmp = g_strdup_printf ("%s/../Resources", app_dir);
+      res_dir = g_canonicalize_filename (tmp, NULL);
       g_free (tmp);
-      tmp = g_strdup_printf ("%s/../Resources/share", bin_dir);
-      share_dir = g_canonicalize_filename (tmp, NULL);
-      g_free (tmp);
-      tmp = g_strdup_printf ("%s/../Resources/etc", bin_dir);
-      etc_dir = g_canonicalize_filename (tmp, NULL);
-      g_free (tmp);
-
-      /* Detect if we are running from bundle or from prefix */
-      if (share_dir && !stat (share_dir, &sb) && S_ISDIR (sb.st_mode))
+      if (res_dir && !stat (res_dir, &sb) && S_ISDIR (sb.st_mode))
         {
           g_print ("GIMP is started as MacOS application\n");
         }
       else
         {
-          tmp = g_strdup_printf ("%s/../share", bin_dir);
-          share_dir = g_canonicalize_filename (tmp, NULL);
+          tmp = g_strdup_printf ("%s/../share", app_dir);
+          res_dir = g_canonicalize_filename (tmp, NULL);
           g_free (tmp);
-          if (share_dir && !stat (share_dir, &sb) && S_ISDIR (sb.st_mode))
+          if (res_dir && !stat (res_dir, &sb) && S_ISDIR (sb.st_mode))
             {
-              g_free (share_dir);
+              g_free (res_dir);
 
               g_print ("GIMP is started in the build directory\n");
 
-              tmp = g_strdup_printf ("%s/..", bin_dir); /* running in build dir */
-              share_dir = g_canonicalize_filename (tmp, NULL);
+              tmp = g_strdup_printf ("%s/..", app_dir); /* running in build dir */
+              res_dir = g_canonicalize_filename (tmp, NULL);
               g_free (tmp);
             }
           else
             {
-              g_free (share_dir);
+              g_free (res_dir);
               return;
             }
         }
 
-      /* Detect we were built in homebrew for MacOS (for PYTHONHOME purposes) */
-      tmp = g_strdup_printf ("%s/../Frameworks/Python.framework", share_dir);
+      /* Detect we were built in homebrew for MacOS */
+      tmp = g_strdup_printf ("%s/Frameworks/Python.framework", res_dir);
       if (tmp && !stat (tmp, &sb) && S_ISDIR (sb.st_mode))
         {
           g_print ("GIMP was built with homebrew\n");
           need_pythonhome = FALSE;
         }
       g_free (tmp);
-      /* Detect we were built in MacPorts for MacOS (for PYTHONHOME purposes) */
-      tmp = g_strdup_printf ("%s/../Library/Frameworks/Python.framework", share_dir);
+      /* Detect we were built in MacPorts for MacOS */
+      tmp = g_strdup_printf ("%s/Library/Frameworks/Python.framework", res_dir);
       if (tmp && !stat (tmp, &sb) && S_ISDIR (sb.st_mode))
         {
           g_print ("GIMP was built with MacPorts\n");
@@ -409,8 +387,7 @@ gimp_macos_setenv (const char * progname)
         }
       g_free (tmp);
 
-      /* Minimum runtime paths */
-      path_len = strlen (g_getenv ("PATH") ? g_getenv ("PATH") : "") + strlen (bin_dir) + 2;
+      path_len = strlen (g_getenv ("PATH") ? g_getenv ("PATH") : "") + strlen (app_dir) + 2;
       path = g_try_malloc (path_len);
       if (path == NULL)
         {
@@ -418,71 +395,59 @@ gimp_macos_setenv (const char * progname)
           app_exit (EXIT_FAILURE);
         }
       if (g_getenv ("PATH"))
-        g_snprintf (path, path_len, "%s:%s", bin_dir, g_getenv ("PATH"));
+        g_snprintf (path, path_len, "%s:%s", app_dir, g_getenv ("PATH"));
       else
-        g_snprintf (path, path_len, "%s", bin_dir);
-      g_free (bin_dir);
+        g_snprintf (path, path_len, "%s", app_dir);
+      g_free (app_dir);
       g_setenv ("PATH", path, TRUE);
       g_free (path);
-      tmp = g_strdup_printf ("%s/gegl-0.4", lib_dir);
-      g_setenv ("GEGL_PATH", tmp, TRUE);
-      g_free (tmp);
-      if (g_getenv ("XDG_DATA_DIRS"))
-        tmp = g_strdup_printf ("%s:%s", share_dir, g_getenv ("XDG_DATA_DIRS"));
-      else
-        tmp = g_strdup_printf ("%s", share_dir);
-      g_setenv ("XDG_DATA_DIRS", tmp, TRUE);
-      g_free (tmp);
-      if (g_getenv ("HOME") != NULL)
-        {
-          tmp = g_strdup_printf ("%s/Library/Application Support/GIMP/%s/cache",
-                                 g_getenv ("HOME"), GIMP_APP_VERSION);
-          g_setenv ("XDG_CACHE_HOME", tmp, TRUE);
-          g_free (tmp);
-        }
-
-      /* Bare minimum to run GTK apps */
-      tmp = g_strdup_printf ("%s/gio/modules", lib_dir);
-      g_setenv ("GIO_MODULE_DIR", tmp, TRUE);
-      g_free (tmp);
-      tmp = g_strdup_printf ("%s/gdk-pixbuf-2.0/2.10.0/loaders.cache", lib_dir);
-      g_setenv ("GDK_PIXBUF_MODULE_FILE", tmp, TRUE);
-      g_free (tmp);
-      tmp = g_strdup_printf ("%s/gtk-3.0/3.0.0", lib_dir);
+      tmp = g_strdup_printf ("%s/lib/gtk-3.0/3.0.0", res_dir);
       g_setenv ("GTK_PATH", tmp, TRUE);
       g_free (tmp);
-      tmp = g_strdup_printf ("%s/gtk-3.0/3.0.0/immodules.cache", lib_dir);
-      g_setenv ("GTK_IM_MODULE_FILE", tmp, TRUE);
+      tmp = g_strdup_printf ("%s/lib/gegl-0.4", res_dir);
+      g_setenv ("GEGL_PATH", tmp, TRUE);
       g_free (tmp);
-      tmp = g_strdup ("quartz");
-      g_setenv ("GTK_IM_MODULE", tmp, TRUE);
+      tmp = g_strdup_printf ("%s/lib/babl-0.1", res_dir);
+      g_setenv ("BABL_PATH", tmp, TRUE);
       g_free (tmp);
-
-      /* Other needed runtime paths (related to features) */
-      tmp = g_strdup_printf ("%s/fonts", etc_dir);
+      tmp = g_strdup_printf ("%s/lib/gdk-pixbuf-2.0/2.10.0/loaders.cache", res_dir);
+      g_setenv ("GDK_PIXBUF_MODULE_FILE", tmp, TRUE);
+      g_free (tmp);
+      tmp = g_strdup_printf ("%s/etc/fonts", res_dir);
       g_setenv ("FONTCONFIG_PATH", tmp, TRUE);
       g_free (tmp);
-      tmp = g_strdup_printf ("%s/libthai", share_dir);
-      g_setenv ("LIBTHAI_DICTDIR", tmp, TRUE);
-      g_free (tmp);
-      tmp = g_strdup_printf ("%s/ghostscript/Resource/Init", share_dir);
-      g_setenv ("GS_LIB", tmp, TRUE);
-      g_free (tmp);
-      tmp = g_strdup_printf ("%s/libwmf/fonts", share_dir);
-      g_setenv ("WMF_FONTDIR", tmp, TRUE);
-      g_free (tmp);
-      tmp = g_strdup_printf ("%s/girepository-1.0", lib_dir);
-      g_setenv ("GI_TYPELIB_PATH", tmp, TRUE);
-      g_free (tmp);
       if (need_pythonhome)
         {
-          tmp = g_strdup_printf ("%s/Library/Frameworks/Python.framework/Versions/%s", share_dir, PYTHON_VERSION);
+          tmp = g_strdup_printf ("%s", res_dir);
           g_setenv ("PYTHONHOME", tmp, TRUE);
           g_free (tmp);
         }
-      g_free (lib_dir);
-      g_free (share_dir);
-      g_free (etc_dir);
+      tmp = g_strdup_printf ("%s/lib/python3.9", res_dir);
+      g_setenv ("PYTHONPATH", tmp, TRUE);
+      g_free (tmp);
+      tmp = g_strdup_printf ("%s/lib/gio/modules", res_dir);
+      g_setenv ("GIO_MODULE_DIR", tmp, TRUE);
+      g_free (tmp);
+      tmp = g_strdup_printf ("%s/share/libwmf/fonts", res_dir);
+      g_setenv ("WMF_FONTDIR", tmp, TRUE);
+      g_free (tmp);
+      if (g_getenv ("XDG_DATA_DIRS"))
+        tmp = g_strdup_printf ("%s/share:%s", res_dir, g_getenv ("XDG_DATA_DIRS"));
+      else
+        tmp = g_strdup_printf ("%s/share", res_dir);
+      g_setenv ("XDG_DATA_DIRS", tmp, TRUE);
+      g_free (tmp);
+      tmp = g_strdup_printf ("%s/lib/girepository-1.0", res_dir);
+      g_setenv ("GI_TYPELIB_PATH", tmp, TRUE);
+      g_free (tmp);
+      if (g_getenv ("HOME") != NULL)
+        {
+          tmp = g_strdup_printf ("%s/Library/Application Support/GIMP/3.00/cache",
+                                 g_getenv ("HOME"));
+          g_setenv ("XDG_CACHE_HOME", tmp, TRUE);
+          g_free (tmp);
+        }
+      g_free (res_dir);
     }
   g_free (resolved_path);
 }
@@ -580,21 +545,21 @@ main (int    argc,
       char **argv)
 {
   GOptionContext *context;
+  GError         *error = NULL;
   const gchar    *abort_message;
   GFile          *system_gimprc_file = NULL;
   GFile          *user_gimprc_file   = NULL;
   GOptionGroup   *gimp_group         = NULL;
   gchar          *backtrace_file     = NULL;
-  GError         *error              = NULL;
-#ifdef G_OS_WIN32
-  gchar          *utf8_name;
-  wchar_t        *name;
+#ifndef GIMP_CONSOLE_COMPILATION
+  GKeyFile       *flatpak_keyfile;
 #endif
   gint            retval;
   gint            i;
 
-  gimp_attach_console_window ();
-
+#ifdef ENABLE_WIN32_DEBUG_CONSOLE
+  gimp_open_console_window ();
+#endif
 #if defined(ENABLE_RELOCATABLE_RESOURCES) && defined(__APPLE__)
   /* remove MacOS session identifier from the command line args */
   gint newargc = 0;
@@ -606,7 +571,6 @@ main (int    argc,
           newargc++;
         }
     }
-
   if (argc > newargc)
     {
       argv[newargc] = NULL; /* glib expects NULL terminated array */
@@ -637,33 +601,28 @@ main (int    argc,
 
 #ifdef G_OS_WIN32
   /* Make Inno aware of gimp process avoiding broken install/unninstall */
-  utf8_name = g_strdup_printf ("GIMP-%s", GIMP_MUTEX_VERSION);
-  name      = g_utf8_to_utf16 (utf8_name, -1, NULL, NULL, NULL);
-
+  char    *utf8_name = g_strdup_printf ("GIMP-%s", GIMP_MUTEX_VERSION);
+  wchar_t *name      = g_utf8_to_utf16 (utf8_name, -1, NULL, NULL, NULL);
+  
   CreateMutexW (NULL, FALSE, name);
-
+  
   g_free (utf8_name);
   g_free (name);
-
+  
   /* Enable Anti-Aliasing*/
   g_setenv ("PANGOCAIRO_BACKEND", "fc", TRUE);
 
   /* Reduce risks */
   SetDllDirectoryW (L"");
 
+  /* On Windows, set DLL search path to $INSTALLDIR/bin so that .exe
+     plug-ins in the plug-ins directory can find libgimp and file
+     library DLLs without needing to set external PATH. */
   {
     const gchar *install_dir;
     gchar       *bin_dir;
     LPWSTR       w_bin_dir;
-#ifdef ENABLE_RELOCATABLE_RESOURCES
-    size_t       path_len;
-    gchar       *path;
-#endif
 
-    /* On Windows, set DLL search path to $INSTALLDIR/bin so that .exe
-     * plug-ins processes in the plug-ins directory can find libgimp and
-     * file library DLLs.
-     */
     w_bin_dir = NULL;
     install_dir = gimp_installation_directory ();
     bin_dir = g_build_filename (install_dir, "bin", NULL);
@@ -675,25 +634,6 @@ main (int    argc,
         g_free (w_bin_dir);
       }
 
-#ifdef ENABLE_RELOCATABLE_RESOURCES
-    /* We also set external PATH variable to other processes just in case
-     * (see #14716 and many other issues labeled as 'DLL Hell').
-     */
-    path_len = strlen (g_getenv ("PATH") ? g_getenv ("PATH") : "") + strlen (bin_dir) + 2;
-    path = g_try_malloc (path_len);
-    if (path == NULL)
-      {
-        g_warning ("Failed to allocate memory");
-        app_exit (EXIT_FAILURE);
-      }
-    if (g_getenv ("PATH"))
-      g_snprintf (path, path_len, "%s;%s", bin_dir, g_getenv ("PATH"));
-    else
-      g_snprintf (path, path_len, "%s", bin_dir);
-    g_setenv ("PATH", path, TRUE);
-    g_free (path);
-#endif
-
     g_free (bin_dir);
   }
 
@@ -706,8 +646,7 @@ main (int    argc,
       (t_SetProcessDEPPolicy) GetProcAddress (GetModuleHandleW (L"kernel32.dll"),
                                               "SetProcessDEPPolicy");
     if (p_SetProcessDEPPolicy)
-      (*p_SetProcessDEPPolicy) (PROCESS_DEP_ENABLE |
-                                PROCESS_DEP_DISABLE_ATL_THUNK_EMULATION);
+      (*p_SetProcessDEPPolicy) (PROCESS_DEP_ENABLE|PROCESS_DEP_DISABLE_ATL_THUNK_EMULATION);
   }
 #endif
 
@@ -777,7 +716,7 @@ main (int    argc,
                (strcmp (arg, "-?") == 0) ||
                (strncmp (arg, "--help-", 7) == 0))
         {
-          gimp_attach_console_window ();
+          gimp_open_console_window ();
         }
 #endif
     }
@@ -791,16 +730,14 @@ main (int    argc,
 
   g_option_context_add_main_entries (context, main_entries, GETTEXT_PACKAGE);
 
-  /* The GIMP option group is just an empty option group, created for
-   * the sole purpose of running a post-parse hook before any other of
-   * dependant libraries are run. This makes it possible to apply
-   * options from configuration data obtained from "gimprc" files,
-   * before other libraries have a chance to run some of their
-   * intialization code.
+  /* The GIMP option group is just an empty option group, created for the sole
+   * purpose of running a post-parse hook before any other of dependant libraries
+   * are run. This makes it possible to apply options from configuration data
+   * obtained from "gimprc" files, before other libraries have a chance to run
+   * some of their intialization code.
    */
   gimp_group = g_option_group_new ("gimp", "", "", NULL, NULL);
-  g_option_group_set_parse_hooks (gimp_group, NULL,
-                                  gimp_options_group_parse_hook);
+  g_option_group_set_parse_hooks (gimp_group, NULL, gimp_options_group_parse_hook);
   g_option_context_add_group (context, gimp_group);
 
   app_libs_init (context, no_interface);
@@ -809,7 +746,7 @@ main (int    argc,
     {
       if (error)
         {
-          gimp_attach_console_window ();
+          gimp_open_console_window ();
           g_print ("%s\n", error->message);
           g_error_free (error);
         }
@@ -827,51 +764,47 @@ main (int    argc,
 #if GLIB_CHECK_VERSION(2,72,0)
   /* g_set_prgname() can only be called several times since 2.72.0. */
 #ifndef GIMP_CONSOLE_COMPILATION
-  if (! no_interface)
-    {
-      GKeyFile *flatpak_keyfile;
+  g_return_val_if_fail (gdk_display_get_default () != NULL, EXIT_FAILURE);
 
-      g_return_val_if_fail (gdk_display_get_default () != NULL, EXIT_FAILURE);
+  flatpak_keyfile = g_key_file_new ();
 
-      flatpak_keyfile = g_key_file_new ();
-
-      if (
+  if (
 #ifdef GDK_WINDOWING_X11
-          ! GDK_IS_X11_DISPLAY (gdk_display_get_default ()) &&
+      ! GDK_IS_X11_DISPLAY (gdk_display_get_default ()) &&
 #endif
-          g_key_file_load_from_file (flatpak_keyfile, "/.flatpak-info",
-                                     G_KEY_FILE_NONE, NULL))
+      g_key_file_load_from_file (flatpak_keyfile, "/.flatpak-info", G_KEY_FILE_NONE, NULL))
+    {
+      /* Flatpak renames the desktop file. The .flatpak-info file tells
+       * us the right desktop name we must associate our process to,
+       * especially as we have flatpaks with different IDs.
+       *
+       * This logic should not apply on X11 which will instead
+       * apparently use the StartupWMClass set in the desktop file and
+       * expect it to be the same as the name set by g_set_prgname().
+       *
+       * Cf. #13183 and #14233.
+       */
+      gchar *flatpak_name = g_key_file_get_string (flatpak_keyfile, "Application", "name", NULL);
+
+      if (flatpak_name != NULL)
         {
-          /* Flatpak renames the desktop file. The .flatpak-info file
-           * tells us the right desktop name we must associate our process
-           * to, especially as we have flatpaks with different IDs.
-           *
-           * This logic should not apply on X11 which will instead
-           * apparently use the StartupWMClass set in the desktop file and
-           * expect it to be the same as the name set by g_set_prgname().
-           *
-           * Cf. #13183 and #14233.
-           */
-          gchar *flatpak_name = g_key_file_get_string (flatpak_keyfile,
-                                                       "Application", "name", NULL);
-
-          if (flatpak_name != NULL)
-            {
-              g_set_prgname (flatpak_name);
-              g_free (flatpak_name);
-            }
-          /* The else case should never happen unless we are in some kind
-           * of broken flatpak environment or somehow in a non-flatpak
-           * environment with a .flatpak-info file at the root, which
-           * seems improbable. Fail silently.
-           */
+          g_set_prgname (flatpak_name);
+          g_free (flatpak_name);
         }
-
-      g_key_file_free (flatpak_keyfile);
+      /* The else case should never happen unless we are in some kind of
+       * broken flatpak environment or somehow in a non-flatpak
+       * environment with a .flatpak-info file at the root, which seems
+       * improbable. Fail silently.
+       */
     }
+
+  g_key_file_free (flatpak_keyfile);
 #endif
 #endif
 
+  if (no_interface || be_verbose || console_messages || batch_commands != NULL)
+    gimp_open_console_window ();
+
   if (no_interface)
     new_instance = TRUE;
 
@@ -949,11 +882,17 @@ main (int    argc,
  * used on MSVC builds only.
  */
 
-int WINAPI
-WinMain (HINSTANCE hInstance,
-         HINSTANCE hPrevInstance,
-         LPSTR     lpCmdLine,
-         int       nCmdShow)
+#ifdef __GNUC__
+#  ifndef _stdcall
+#    define _stdcall  __attribute__((stdcall))
+#  endif
+#endif
+
+int _stdcall
+WinMain (struct HINSTANCE__ *hInstance,
+         struct HINSTANCE__ *hPrevInstance,
+         char               *lpszCmdLine,
+         int                 nCmdShow)
 {
   return main (__argc, __argv);
 }
@@ -963,37 +902,27 @@ WinMain (HINSTANCE hInstance,
 static void
 wait_console_window (void)
 {
-  g_print (_ ("(Type any character to close this window)\n"));
+  FILE *console = g_fopen ("CONOUT$", "w");
+
+  SetConsoleTitleW (g_utf8_to_utf16 (_("GIMP output. Type any character to close this window."), -1, NULL, NULL, NULL));
+  fprintf (console, _("(Type any character to close this window)\n"));
+  fflush (console);
+  _getch ();
 }
 
 static void
-gimp_attach_console_window (void)
+gimp_open_console_window (void)
 {
-  /* If run on non-native shell, do nothing */
-  if (g_getenv ("TERM") || g_getenv ("SHELL"))
+  if (((HANDLE) _get_osfhandle (fileno (stdout)) == INVALID_HANDLE_VALUE ||
+       (HANDLE) _get_osfhandle (fileno (stderr)) == INVALID_HANDLE_VALUE) && AllocConsole ())
     {
-      g_printerr ("Non-native shell detected, GIMP may "
-                  "behave unexpectedly on Unix shells in Windows.\n");
-      return;
-    }
+      if ((HANDLE) _get_osfhandle (fileno (stdout)) == INVALID_HANDLE_VALUE)
+        freopen ("CONOUT$", "w", stdout);
 
-  /* If run on native shell, attach to it */
-  if (AttachConsole (ATTACH_PARENT_PROCESS) != 0)
-    {
-      /* 'r' is needed to prevent interleaving and '+' to support colors */
-      freopen ("CONOUT$", "r+", stdout);
-      freopen ("CONOUT$", "r+", stderr);
-      _flushall ();
+      if ((HANDLE) _get_osfhandle (fileno (stderr)) == INVALID_HANDLE_VALUE)
+        freopen ("CONOUT$", "w", stderr);
 
-      {
-        /* CTRL+C handling */
-        HANDLE hIn = GetStdHandle (STD_INPUT_HANDLE);
-        DWORD  mode;
-
-        GetConsoleMode (hIn, &mode);
-        mode |= ENABLE_PROCESSED_INPUT;
-        SetConsoleMode (hIn, mode);
-      }
+      SetConsoleTitleW (g_utf8_to_utf16 (_("GIMP output. You can minimize this window, but don't close it."), -1, NULL, NULL, NULL));
 
       atexit (wait_console_window);
     }
@@ -1063,7 +992,7 @@ gimp_option_dump_gimprc (const gchar  *option_name,
 {
   GimpConfigDumpFormat format = GIMP_CONFIG_DUMP_NONE;
 
-  gimp_attach_console_window ();
+  gimp_open_console_window ();
 
   if (strcmp (option_name, "--dump-gimprc") == 0)
     format = GIMP_CONFIG_DUMP_GIMPRC;
@@ -1135,7 +1064,7 @@ gimp_option_dump_pdb_procedures_deprecated (const gchar  *option_name,
 static void
 gimp_show_version_and_exit (void)
 {
-  gimp_attach_console_window ();
+  gimp_open_console_window ();
   gimp_version_show (be_verbose);
 
   app_exit (EXIT_SUCCESS);
@@ -1144,10 +1073,12 @@ gimp_show_version_and_exit (void)
 static void
 gimp_show_license_and_exit (void)
 {
-  gimp_attach_console_window ();
+  gimp_open_console_window ();
   gimp_version_show (be_verbose);
 
-  g_print ("\n%s\n\n", GIMP_LICENSE);
+  g_print ("\n");
+  g_print (GIMP_LICENSE);
+  g_print ("\n\n");
 
   app_exit (EXIT_SUCCESS);
 }
