@@ -992,3 +992,98 @@ gimp_levels_config_save_cruft (GimpLevelsConfig  *config,
 
   return TRUE;
 }
+
+gboolean
+gimp_levels_config_load_alv (GimpLevelsConfig  *config,
+                             GInputStream      *input,
+                             GError           **error)
+{
+  GDataInputStream *data_input;
+  guint16           version = 0;
+  gint              low_input[4];
+  gint              high_input[4];
+  gint              low_output[4];
+  gint              high_output[4];
+  gdouble           gamma[4];
+
+  g_return_val_if_fail (GIMP_IS_LEVELS_CONFIG (config), FALSE);
+  g_return_val_if_fail (G_IS_INPUT_STREAM (input), FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  data_input = g_data_input_stream_new (input);
+
+  g_data_input_stream_set_byte_order (data_input,
+                                      G_DATA_STREAM_BYTE_ORDER_BIG_ENDIAN);
+
+  version = g_data_input_stream_read_uint16 (data_input, NULL, error);
+  if (! version)
+    goto error;
+
+  /* TODO: Photoshop Level presets store 29 levels, for channels beyond
+   * RGB. When we can support those, we should update the read limit */
+  for (gint i = 0; i < 4; i++)
+    {
+      low_input[i] = g_data_input_stream_read_uint16 (data_input, NULL, error);
+      if (error && *error)
+        goto error;
+
+      high_input[i] = g_data_input_stream_read_uint16 (data_input, NULL,
+                                                       error);
+      if (error && *error)
+        goto error;
+
+      low_output[i] = g_data_input_stream_read_uint16 (data_input, NULL,
+                                                       error);
+      if (error && *error)
+        goto error;
+
+      high_output[i] = g_data_input_stream_read_uint16 (data_input, NULL,
+                                                        error);
+      if (error && *error)
+        goto error;
+
+      gamma[i] = g_data_input_stream_read_int16 (data_input, NULL, error);
+      if (error && *error)
+        goto error;
+    }
+
+  g_object_unref (data_input);
+
+  g_object_freeze_notify (G_OBJECT (config));
+
+  for (gint i = 0; i < 4; i++)
+    {
+      config->low_input[i]   = low_input[i]   / 253.0;
+      config->high_input[i]  = high_input[i]  / 255.0;
+      config->gamma[i]       = gamma[i]       / 100.0;
+      config->low_output[i]  = low_output[i]  / 255.0;
+      config->high_output[i] = high_output[i] / 255.0;
+    }
+
+  config->trc          = GIMP_TRC_NON_LINEAR;
+  config->clamp_input  = TRUE;
+  config->clamp_output = TRUE;
+
+  g_object_notify (G_OBJECT (config), "trc");
+  g_object_notify (G_OBJECT (config), "low-input");
+  g_object_notify (G_OBJECT (config), "high-input");
+  g_object_notify (G_OBJECT (config), "clamp-input");
+  g_object_notify (G_OBJECT (config), "gamma");
+  g_object_notify (G_OBJECT (config), "low-output");
+  g_object_notify (G_OBJECT (config), "high-output");
+  g_object_notify (G_OBJECT (config), "clamp-output");
+
+  g_object_thaw_notify (G_OBJECT (config));
+
+  return TRUE;
+
+  error:
+    g_object_unref (data_input);
+
+    if (error && *error)
+      g_prefix_error (error, _("parse error"));
+    else
+      g_set_error (error, GIMP_DATA_ERROR, GIMP_DATA_ERROR_READ,
+                   _("parse error"));
+  return FALSE;
+}
