@@ -134,7 +134,8 @@ static GimpTempBuf * gimp_font_get_new_preview    (GimpViewable          *viewab
                                                    GimpContext           *context,
                                                    gint                   width,
                                                    gint                   height,
-                                                   GeglColor             *fg_color);
+                                                   GeglColor             *color,
+                                                   GeglColor             *background);
 
 static void          gimp_font_config_iface_init  (GimpConfigInterface  *iface);
 static gboolean      gimp_font_serialize          (GimpConfig           *config,
@@ -600,7 +601,6 @@ gimp_font_class_init (GimpFontClass *klass)
   viewable_class->get_new_preview   = gimp_font_get_new_preview;
 
   viewable_class->default_icon_name = "gtk-select-font";
-  viewable_class->default_name      = _("Font");
 }
 
 static void
@@ -713,7 +713,8 @@ gimp_font_get_new_preview (GimpViewable *viewable,
                            GimpContext  *context,
                            gint          width,
                            gint          height,
-                           GeglColor    *fg_color)
+                           GeglColor    *color,
+                           GeglColor    *background)
 {
   GimpFont        *font = GIMP_FONT (viewable);
   PangoContext    *pango_context;
@@ -734,9 +735,8 @@ gimp_font_get_new_preview (GimpViewable *viewable,
   if (! pango_context)
     return NULL;
 
-  if (! font->popup_layout        ||
-      font->popup_width  != width ||
-      font->popup_height != height)
+  if (! font->popup_layout ||
+      font->popup_width != width || font->popup_height != height)
     {
       PangoFontDescription *font_desc;
       const gchar          *name;
@@ -769,7 +769,19 @@ gimp_font_get_new_preview (GimpViewable *viewable,
   stride = cairo_format_stride_for_width (CAIRO_FORMAT_ARGB32, width);
 
   temp_buf = gimp_temp_buf_new (width, height, babl_format ("cairo-ARGB32"));
-  memset (gimp_temp_buf_get_data (temp_buf), 0, width * height * 4);
+  if (background)
+    {
+      guint32 *data = (guint32 *) gimp_temp_buf_get_data (temp_buf);
+      guint32  argb;
+
+      gegl_color_get_pixel (background, babl_format ("cairo-ARGB32"), &argb);
+      for (gint i = 0; i < width * height; i++)
+        data[i] = argb;
+    }
+  else
+    {
+      memset (gimp_temp_buf_get_data (temp_buf), 255, width * height * 4);
+    }
 
   surface = cairo_image_surface_create_for_data (gimp_temp_buf_get_data (temp_buf),
                                                  CAIRO_FORMAT_ARGB32,
@@ -793,11 +805,11 @@ gimp_font_get_new_preview (GimpViewable *viewable,
 
   cairo_translate (cr, layout_x, layout_y);
   cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
-  if (fg_color)
+  if (color)
     {
       gfloat rgb[3];
 
-      gegl_color_get_pixel (fg_color, babl_format ("R'G'B' float"), rgb);
+      gegl_color_get_pixel (color, babl_format ("R'G'B' float"), rgb);
       cairo_set_source_rgb (cr, rgb[0], rgb[1], rgb[2]);
     }
   else
@@ -1190,7 +1202,10 @@ gimp_font_get_sample_string (PangoContext         *context,
    */
   hb_font = hb_font_create_sub_font (hb_font);
   hb_ft_font_set_funcs (hb_font);
-  face = hb_ft_font_lock_face (hb_font);
+  /* TODO: use hb_ft_font_lock_face/hb_ft_font_unlock_face() when we
+   * bump to harfbuzz >= 2.6.5.
+   */
+  face = hb_ft_font_get_face (hb_font);
 
   /* Are there actual cases where this function could return NULL while
    * it's not a bug in the code?
@@ -1278,7 +1293,6 @@ gimp_font_get_sample_string (PangoContext         *context,
         }
     }
 
-  hb_ft_font_unlock_face (hb_font);
   hb_font_destroy (hb_font);
   g_object_unref (font);
 
@@ -1349,8 +1363,6 @@ gimp_font_get_sample_string (PangoContext         *context,
 static const gchar *
 gimp_font_get_hash (GimpFont *font)
 {
-  g_return_val_if_fail (font, NULL);
-
   /* Computing the hash is expensive, so it's only done in a few case, such as
    * when serializing, when a similar fonts at deserialization looks like it
    * could be a good identity candidate.
@@ -1367,16 +1379,6 @@ gimp_font_get_hash (GimpFont *font)
       guint                 length;
       const char           *hb_data;
 
-      /* Some people had a crash in pango_fc_font_get_pattern() though
-       * we didn't have anyone providing further test results. The
-       * theory right now is that pango_font was NULL. Adding some
-       * CRITICAL here will protect from the direct crash, and it should
-       * also trigger better traces with variable listing if ever this
-       * happened again (though it may never happen again, because now
-       * we only run this once fonts are fully loaded).
-       * See #14139.
-       */
-      g_return_val_if_fail (pango_font, NULL);
       FcPatternGetString (pango_fc_font_get_pattern (pango_font), FC_FILE, 0, (FcChar8 **) &file);
 
       hb_blob  = hb_blob_create_from_file (file);
