@@ -85,8 +85,6 @@ static void       gimp_text_layer_set_property   (GObject           *object,
 static gint64     gimp_text_layer_get_memsize    (GimpObject        *object,
                                                   gint64            *gui_size);
 
-static void       gimp_text_layer_size_changed   (GimpViewable      *viewable);
-
 static GimpItem * gimp_text_layer_duplicate      (GimpItem          *item,
                                                   GType              new_type);
 static gboolean   gimp_text_layer_rename         (GimpItem          *item,
@@ -145,8 +143,6 @@ gimp_text_layer_class_init (GimpTextLayerClass *klass)
   gimp_object_class->get_memsize    = gimp_text_layer_get_memsize;
 
   viewable_class->default_icon_name = "gimp-text-layer";
-  viewable_class->default_name      = _("Text Layer");
-  viewable_class->size_changed      = gimp_text_layer_size_changed;
 
   item_class->duplicate             = gimp_text_layer_duplicate;
   item_class->rename                = gimp_text_layer_rename;
@@ -158,6 +154,7 @@ gimp_text_layer_class_init (GimpTextLayerClass *klass)
   item_class->transform             = gimp_text_layer_transform;
 #endif
 
+  item_class->default_name          = _("Text Layer");
   item_class->rename_desc           = _("Rename Text Layer");
   item_class->translate_desc        = _("Move Text Layer");
   item_class->scale_desc            = _("Scale Text Layer");
@@ -275,19 +272,6 @@ gimp_text_layer_get_memsize (GimpObject *object,
                                                                   gui_size);
 }
 
-static void
-gimp_text_layer_size_changed (GimpViewable *viewable)
-{
-  GimpTextLayer *text_layer = GIMP_TEXT_LAYER (viewable);
-
-  /* TODO: For now, we only let the size_changed () call bubble up to
-   * gimp_drawable_size_changed () if the layer has been rasterized by
-   * a transform. This prevents filters like Drop Shadow from being
-   * cropped just by typing */
-  if (text_layer->modified)
-    GIMP_VIEWABLE_CLASS (parent_class)->size_changed (viewable);
-}
-
 static GimpItem *
 gimp_text_layer_duplicate (GimpItem *item,
                            GType     new_type)
@@ -1030,10 +1014,31 @@ gimp_text_layer_render_layout (GimpTextLayer  *layer,
       cairo_set_line_width (cr, text->outline_width * 2);
 
       gimp_text_layout_render (layout, cr, text->base_dir, TRUE);
-      cairo_clip_preserve (cr);
-      cairo_stroke (cr);
+
+      if (text->outline_direction == GIMP_TEXT_OUTLINE_DIRECTION_INNER)
+        cairo_clip_preserve (cr);
+
+      cairo_stroke_preserve (cr);
+
+      /* Clears inner outline if outline direction is outward */
+      if (text->outline_direction == GIMP_TEXT_OUTLINE_DIRECTION_OUTER &&
+          text->outline == GIMP_TEXT_OUTLINE_STROKE_ONLY)
+        {
+          cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
+          cairo_fill_preserve (cr);
+        }
 
       cairo_restore (cr);
+
+      if (text->outline_direction == GIMP_TEXT_OUTLINE_DIRECTION_OUTER &&
+          text->outline != GIMP_TEXT_OUTLINE_STROKE_ONLY)
+        {
+          cairo_save (cr);
+
+          gimp_text_layout_render (layout, cr, layer->text->base_dir, FALSE);
+
+          cairo_restore (cr);
+        }
     }
 
   cairo_destroy (cr);
@@ -1059,7 +1064,7 @@ gimp_text_layer_render_layout (GimpTextLayer  *layer,
 #else
   format = babl_format_with_space ("cairo-ARGB32", gimp_text_layout_get_space (layout));
 #endif
-  buffer = gimp_cairo_surface_get_buffer (surface, format, FALSE);
+  buffer = gimp_cairo_surface_create_buffer (surface, format);
 
   gimp_gegl_buffer_copy (buffer, NULL, GEGL_ABYSS_NONE,
                          gimp_drawable_get_buffer (drawable), NULL);
