@@ -38,7 +38,6 @@
 #include "gimpcontainertreestore.h"
 #include "gimpcontainericonview.h"
 #include "gimpcontainerview.h"
-#include "gimpcontainerview-cruft.h"
 #include "gimpdnd.h"
 #include "gimpviewrenderer.h"
 #include "gimpwidgets-utils.h"
@@ -66,11 +65,6 @@ static void          gimp_container_icon_view_set_context       (GimpContainerVi
                                                                  GimpContext                 *context);
 static void          gimp_container_icon_view_set_selection_mode(GimpContainerView           *view,
                                                                  GtkSelectionMode             mode);
-static void          gimp_container_icon_view_set_view_size     (GimpContainerView           *view);
-static gboolean      gimp_container_icon_view_set_selected      (GimpContainerView           *view,
-                                                                 GList                       *items);
-static gint          gimp_container_icon_view_get_selected        (GimpContainerView      *view,
-                                                                   GList                 **items);
 
 static gpointer      gimp_container_icon_view_insert_item       (GimpContainerView           *view,
                                                                  GimpViewable                *viewable,
@@ -86,8 +80,11 @@ static void          gimp_container_icon_view_reorder_item      (GimpContainerVi
 static void          gimp_container_icon_view_rename_item       (GimpContainerView           *view,
                                                                  GimpViewable                *viewable,
                                                                  gpointer                     insert_data);
+static gboolean      gimp_container_icon_view_select_items      (GimpContainerView           *view,
+                                                                 GList                       *items,
+                                                                 GList                       *paths);
 static void          gimp_container_icon_view_clear_items       (GimpContainerView           *view);
-
+static void          gimp_container_icon_view_set_view_size     (GimpContainerView           *view);
 static void          gimp_container_icon_view_invalidate        (GimpContainerIconView       *view);
 
 static void          gimp_container_icon_view_selection_changed (GtkIconView                 *view,
@@ -108,12 +105,15 @@ static gboolean      gimp_container_icon_view_tooltip           (GtkWidget
 static GimpViewable * gimp_container_icon_view_drag_viewable    (GtkWidget    *widget,
                                                                  GimpContext **context,
                                                                  gpointer      data);
-static GdkPixbuf   * gimp_container_icon_view_drag_pixbuf       (GtkWidget *widget,
-                                                                 gpointer   data);
-static gboolean      gimp_container_icon_view_get_1_selected    (GimpContainerIconView  *icon_view,
-                                                                 GtkTreeIter            *iter);
+static GdkPixbuf    * gimp_container_icon_view_drag_pixbuf        (GtkWidget *widget,
+                                                                   gpointer   data);
+static gboolean      gimp_container_icon_view_get_selected_single (GimpContainerIconView  *icon_view,
+                                                                   GtkTreeIter            *iter);
+static gint          gimp_container_icon_view_get_selected        (GimpContainerView      *view,
+                                                                   GList                 **items,
+                                                                   GList                 **paths);
 
-static void          gimp_container_icon_view_trigger_redraw    (GimpContainerIconView *view);
+static void          gimp_container_icon_view_trigger_redraw      (GimpContainerIconView *view);
 
 
 G_DEFINE_TYPE_WITH_CODE (GimpContainerIconView, gimp_container_icon_view,
@@ -153,15 +153,14 @@ gimp_container_icon_view_view_iface_init (GimpContainerViewInterface *iface)
   iface->set_container      = gimp_container_icon_view_set_container;
   iface->set_context        = gimp_container_icon_view_set_context;
   iface->set_selection_mode = gimp_container_icon_view_set_selection_mode;
-  iface->set_view_size      = gimp_container_icon_view_set_view_size;
-  iface->set_selected       = gimp_container_icon_view_set_selected;
-  iface->get_selected       = gimp_container_icon_view_get_selected;
-
   iface->insert_item        = gimp_container_icon_view_insert_item;
   iface->remove_item        = gimp_container_icon_view_remove_item;
   iface->reorder_item       = gimp_container_icon_view_reorder_item;
   iface->rename_item        = gimp_container_icon_view_rename_item;
+  iface->select_items       = gimp_container_icon_view_select_items;
   iface->clear_items        = gimp_container_icon_view_clear_items;
+  iface->set_view_size      = gimp_container_icon_view_set_view_size;
+  iface->get_selected       = gimp_container_icon_view_get_selected;
 
   iface->insert_data_free = (GDestroyNotify) gtk_tree_iter_free;
 }
@@ -260,7 +259,7 @@ gimp_container_icon_view_popup_menu (GtkWidget *widget)
   GtkTreePath           *path;
   GdkRectangle           rect;
 
-  if (! gimp_container_icon_view_get_1_selected (icon_view, &iter))
+  if (!gimp_container_icon_view_get_selected_single (icon_view, &iter))
     return FALSE;
 
   path = gtk_tree_model_get_path (icon_view->model, &iter);
@@ -297,12 +296,12 @@ gimp_container_icon_view_new (GimpContainer *container,
 
   gimp_container_view_set_view_size (view, view_size, 0 /* ignore border */);
 
-  if (context)
-    gimp_container_view_set_context (view, context);
-
   if (container)
     gimp_container_view_set_container (view, container);
 
+  if (context)
+    gimp_container_view_set_context (view, context);
+
   return GTK_WIDGET (icon_view);
 }
 
@@ -323,9 +322,9 @@ gimp_container_icon_view_set_container (GimpContainerView *view,
       if (! container)
         {
           if (gimp_dnd_viewable_source_remove (GTK_WIDGET (icon_view->view),
-                                               gimp_container_get_child_type (old_container)))
+                                               gimp_container_get_children_type (old_container)))
             {
-              if (GIMP_VIEWABLE_CLASS (g_type_class_peek (gimp_container_get_child_type (old_container)))->get_size)
+              if (GIMP_VIEWABLE_CLASS (g_type_class_peek (gimp_container_get_children_type (old_container)))->get_size)
                 gimp_dnd_pixbuf_source_remove (GTK_WIDGET (icon_view->view));
 
               gtk_drag_source_unset (GTK_WIDGET (icon_view->view));
@@ -340,15 +339,15 @@ gimp_container_icon_view_set_container (GimpContainerView *view,
     {
       if (gimp_dnd_drag_source_set_by_type (GTK_WIDGET (icon_view->view),
                                             GDK_BUTTON1_MASK | GDK_BUTTON2_MASK,
-                                            gimp_container_get_child_type (container),
+                                            gimp_container_get_children_type (container),
                                             GDK_ACTION_COPY))
         {
           gimp_dnd_viewable_source_add (GTK_WIDGET (icon_view->view),
-                                        gimp_container_get_child_type (container),
+                                        gimp_container_get_children_type (container),
                                         gimp_container_icon_view_drag_viewable,
                                         icon_view);
 
-          if (GIMP_VIEWABLE_CLASS (g_type_class_peek (gimp_container_get_child_type (container)))->get_size)
+          if (GIMP_VIEWABLE_CLASS (g_type_class_peek (gimp_container_get_children_type (container)))->get_size)
             gimp_dnd_pixbuf_source_add (GTK_WIDGET (icon_view->view),
                                         gimp_container_icon_view_drag_pixbuf,
                                         icon_view);
@@ -375,14 +374,13 @@ gimp_container_icon_view_set_context (GimpContainerView *view,
     gimp_container_tree_store_set_context (GIMP_CONTAINER_TREE_STORE (icon_view->model),
                                            context);
 
-  if (context)
+  if (context != NULL)
     {
-      if (! icon_view->priv->color_scheme_handler_id)
-        icon_view->priv->color_scheme_handler_id =
-          g_signal_connect_object (context->gimp->config,
-                                   "notify::theme-color-scheme",
-                                   G_CALLBACK (gimp_container_icon_view_trigger_redraw),
-                                   icon_view, G_CONNECT_SWAPPED);
+      if (icon_view->priv->color_scheme_handler_id == 0)
+        icon_view->priv->color_scheme_handler_id = g_signal_connect_object (context->gimp->config,
+                                                                            "notify::theme-color-scheme",
+                                                                            G_CALLBACK (gimp_container_icon_view_trigger_redraw),
+                                                                            icon_view, G_CONNECT_SWAPPED);
     }
 }
 
@@ -397,136 +395,6 @@ gimp_container_icon_view_set_selection_mode (GimpContainerView *view,
   parent_view_iface->set_selection_mode (view, mode);
 }
 
-static void
-gimp_container_icon_view_set_view_size (GimpContainerView *view)
-{
-  GimpContainerIconView *icon_view = GIMP_CONTAINER_ICON_VIEW (view);
-
-  if (icon_view->model)
-    gimp_container_tree_store_set_view_size (GIMP_CONTAINER_TREE_STORE (icon_view->model));
-
-  if (icon_view->view)
-    {
-      gtk_icon_view_set_columns (icon_view->view, -1);
-      gtk_icon_view_set_item_width (icon_view->view, -1);
-
-      /* ugly workaround to force the icon view to invalidate all its
-       * cached icon sizes
-       */
-      gtk_icon_view_set_item_orientation (icon_view->view,
-                                          GTK_ORIENTATION_VERTICAL);
-      gtk_icon_view_set_item_orientation (icon_view->view,
-                                          GTK_ORIENTATION_HORIZONTAL);
-    }
-}
-
-static gboolean
-gimp_container_icon_view_set_selected (GimpContainerView *view,
-                                       GList             *items)
-{
-  GimpContainerIconView *icon_view = GIMP_CONTAINER_ICON_VIEW (view);
-  GList                 *list;
-
-  if (items)
-    {
-      GList *paths = NULL;
-
-      for (list = items; list; list = g_list_next (list))
-        {
-          GtkTreeIter *iter;
-          GtkTreePath *path;
-
-          iter = _gimp_container_view_lookup (GIMP_CONTAINER_VIEW (view),
-                                              list->data);
-          if (! iter)
-            /* This may happen when the GimpContainerIconView only
-             * shows a subpart of the whole icons. We don't select
-             * what is not shown.
-             */
-            continue;
-
-          path = gtk_tree_model_get_path (icon_view->model, iter);
-
-          paths = g_list_prepend (paths, path);
-        }
-
-      paths = g_list_reverse (paths);
-
-      g_signal_handlers_block_by_func (icon_view->view,
-                                       gimp_container_icon_view_selection_changed,
-                                       icon_view);
-
-      gtk_icon_view_unselect_all (icon_view->view);
-
-      for (list = paths; list; list = g_list_next (list))
-        {
-          gtk_icon_view_select_path (icon_view->view, list->data);
-        }
-
-      if (paths)
-        {
-          gtk_icon_view_set_cursor (icon_view->view, paths->data, NULL, FALSE);
-          gtk_icon_view_scroll_to_path (icon_view->view, paths->data,
-                                        FALSE, 0.0, 0.0);
-        }
-
-      g_signal_handlers_unblock_by_func (icon_view->view,
-                                         gimp_container_icon_view_selection_changed,
-                                         icon_view);
-
-      g_list_free_full (paths, (GDestroyNotify) gtk_tree_path_free);
-
-      _gimp_container_view_selection_changed (GIMP_CONTAINER_VIEW (icon_view));
-    }
-  else
-    {
-      gtk_icon_view_unselect_all (icon_view->view);
-    }
-
-  return TRUE;
-}
-
-static gint
-gimp_container_icon_view_get_selected (GimpContainerView  *view,
-                                       GList             **items)
-{
-  GimpContainerIconView *icon_view = GIMP_CONTAINER_ICON_VIEW (view);
-  GList                 *paths;
-  gint                   n_selected;
-
-  paths = gtk_icon_view_get_selected_items (icon_view->view);
-  n_selected = g_list_length (paths);
-
-  if (items)
-    {
-      GList *list;
-
-      for (list = paths; list; list = g_list_next (list))
-        {
-          GtkTreeIter       iter;
-          GimpViewRenderer *renderer;
-
-          gtk_tree_model_get_iter (GTK_TREE_MODEL (icon_view->model), &iter,
-                                   (GtkTreePath *) list->data);
-
-          gtk_tree_model_get (icon_view->model, &iter,
-                              GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
-                              -1);
-
-          if (renderer->viewable)
-            *items = g_list_prepend (*items, renderer->viewable);
-
-          g_object_unref (renderer);
-        }
-
-      *items = g_list_reverse (*items);
-    }
-
-  g_list_free_full (paths, (GDestroyNotify) gtk_tree_path_free);
-
-  return n_selected;
-}
-
 static gpointer
 gimp_container_icon_view_insert_item (GimpContainerView *view,
                                       GimpViewable      *viewable,
@@ -581,8 +449,8 @@ gimp_container_icon_view_reorder_item (GimpContainerView *view,
     {
       GtkTreeIter selected_iter;
 
-      selected = gimp_container_icon_view_get_1_selected (icon_view,
-                                                          &selected_iter);
+      selected = gimp_container_icon_view_get_selected_single (icon_view,
+                                                               &selected_iter);
 
       if (selected)
         {
@@ -605,7 +473,7 @@ gimp_container_icon_view_reorder_item (GimpContainerView *view,
                                           iter);
 
   if (selected)
-    gimp_container_view_set_1_selected (view, viewable);
+    gimp_container_view_select_item (view, viewable);
 }
 
 static void
@@ -621,6 +489,75 @@ gimp_container_icon_view_rename_item (GimpContainerView *view,
                                          iter);
 }
 
+static gboolean
+gimp_container_icon_view_select_items (GimpContainerView *view,
+                                       GList             *viewables,
+                                       GList             *paths)
+{
+  GimpContainerIconView *icon_view = GIMP_CONTAINER_ICON_VIEW (view);
+  GList                 *list;
+
+  if (viewables)
+    {
+      gboolean free_paths = FALSE;
+
+      if (g_list_length (paths) != g_list_length (viewables))
+        {
+          free_paths = TRUE;
+          paths = NULL;
+          for (list = viewables; list; list = list->next)
+            {
+              GtkTreeIter *iter;
+              GtkTreePath *path;
+
+              iter = gimp_container_view_lookup (GIMP_CONTAINER_VIEW (view),
+                                                 list->data);
+              if (! iter)
+                /* This may happen when the GimpContainerIconView only
+                 * shows a subpart of the whole icons. We don't select
+                 * what is not shown.
+                 */
+                continue;
+
+              path = gtk_tree_model_get_path (icon_view->model, iter);
+
+              paths = g_list_prepend (paths, path);
+            }
+          paths = g_list_reverse (paths);
+        }
+
+      g_signal_handlers_block_by_func (icon_view->view,
+                                       gimp_container_icon_view_selection_changed,
+                                       icon_view);
+
+      gtk_icon_view_unselect_all (icon_view->view);
+
+      for (list = paths; list; list = list->next)
+        {
+          gtk_icon_view_select_path (icon_view->view, list->data);
+        }
+
+      if (list)
+        {
+          gtk_icon_view_set_cursor (icon_view->view, list->data, NULL, FALSE);
+          gtk_icon_view_scroll_to_path (icon_view->view, list->data, FALSE, 0.0, 0.0);
+        }
+
+      g_signal_handlers_unblock_by_func (icon_view->view,
+                                         gimp_container_icon_view_selection_changed,
+                                         icon_view);
+
+      if (free_paths)
+        g_list_free_full (paths, (GDestroyNotify) gtk_tree_path_free);
+    }
+  else
+    {
+      gtk_icon_view_unselect_all (icon_view->view);
+    }
+
+  return TRUE;
+}
+
 static void
 gimp_container_icon_view_clear_items (GimpContainerView *view)
 {
@@ -631,6 +568,29 @@ gimp_container_icon_view_clear_items (GimpContainerView *view)
   parent_view_iface->clear_items (view);
 }
 
+static void
+gimp_container_icon_view_set_view_size (GimpContainerView *view)
+{
+  GimpContainerIconView *icon_view = GIMP_CONTAINER_ICON_VIEW (view);
+
+  if (icon_view->model)
+    gimp_container_tree_store_set_view_size (GIMP_CONTAINER_TREE_STORE (icon_view->model));
+
+  if (icon_view->view)
+    {
+      gtk_icon_view_set_columns (icon_view->view, -1);
+      gtk_icon_view_set_item_width (icon_view->view, -1);
+
+      /* ugly workaround to force the icon view to invalidate all its
+       * cached icon sizes
+       */
+      gtk_icon_view_set_item_orientation (icon_view->view,
+                                          GTK_ORIENTATION_VERTICAL);
+      gtk_icon_view_set_item_orientation (icon_view->view,
+                                          GTK_ORIENTATION_HORIZONTAL);
+    }
+}
+
 static void
 gimp_container_icon_view_invalidate (GimpContainerIconView *view)
 {
@@ -660,7 +620,35 @@ static void
 gimp_container_icon_view_selection_changed (GtkIconView           *gtk_icon_view,
                                             GimpContainerIconView *icon_view)
 {
-  _gimp_container_view_selection_changed (GIMP_CONTAINER_VIEW (icon_view));
+  GimpContainerView *view = GIMP_CONTAINER_VIEW (icon_view);
+  GList             *items = NULL;
+  GList             *paths;
+  GList             *list;
+
+  paths = gtk_icon_view_get_selected_items (icon_view->view);
+
+  for (list = paths; list; list = list->next)
+    {
+      GtkTreeIter       iter;
+      GimpViewRenderer *renderer;
+
+      gtk_tree_model_get_iter (GTK_TREE_MODEL (icon_view->model), &iter,
+                               (GtkTreePath *) list->data);
+
+      gtk_tree_model_get (icon_view->model, &iter,
+                          GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
+                          -1);
+
+      if (renderer->viewable)
+        items = g_list_prepend (items, renderer->viewable);
+
+      g_object_unref (renderer);
+    }
+
+  gimp_container_view_multi_selected (view, items, paths);
+
+  g_list_free_full (paths, (GDestroyNotify) gtk_tree_path_free);
+  g_list_free (items);
 }
 
 static void
@@ -677,8 +665,8 @@ gimp_container_icon_view_item_activated (GtkIconView           *view,
                       GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
                       -1);
 
-  _gimp_container_view_item_activated (GIMP_CONTAINER_VIEW (icon_view),
-                                       renderer->viewable);
+  gimp_container_view_item_activated (GIMP_CONTAINER_VIEW (icon_view),
+                                      renderer->viewable);
 
   g_object_unref (renderer);
 }
@@ -716,16 +704,11 @@ gimp_container_icon_view_button_press (GtkWidget             *widget,
            * selection. Otherwise, we use the current selection. This
            * allows to not break multiple selection when right-clicking.
            */
-          if (! gimp_container_view_is_item_selected (container_view,
-                                                      renderer->viewable))
-            {
-              gimp_container_view_set_1_selected (container_view,
-                                                  renderer->viewable);
-            }
+          if (! gimp_container_view_is_item_selected (container_view, renderer->viewable))
+            gimp_container_view_item_selected (container_view, renderer->viewable);
 
           /* Show the context menu. */
-          handled = gimp_editor_popup_menu_at_pointer (GIMP_EDITOR (icon_view),
-                                                       (GdkEvent *) bevent);
+          handled = gimp_editor_popup_menu_at_pointer (GIMP_EDITOR (icon_view), (GdkEvent *) bevent);
         }
       /* Else LMB down or similar.  Propagate. */
 
@@ -738,8 +721,7 @@ gimp_container_icon_view_button_press (GtkWidget             *widget,
       /* Button down outside any item. */
       if (gdk_event_triggers_context_menu ((GdkEvent *) bevent))
         {
-          (void) gimp_editor_popup_menu_at_pointer (GIMP_EDITOR (icon_view),
-                                                    (GdkEvent *) bevent);
+          (void) gimp_editor_popup_menu_at_pointer (GIMP_EDITOR (icon_view), (GdkEvent *) bevent);
           /* Discard result.  Does not actually popup menu except for managed views, e.g. dockable. */
         }
 
@@ -828,16 +810,62 @@ gimp_container_icon_view_drag_pixbuf (GtkWidget *widget,
   gint                   height;
 
   if (renderer && gimp_viewable_get_size (renderer->viewable, &width, &height))
-    return gimp_viewable_get_new_pixbuf (renderer->viewable,
-                                         renderer->context,
-                                         width, height, NULL);
+    {
+      GeglColor *color      = NULL;
+      GeglColor *background = NULL;
+      GdkPixbuf *pixbuf;
+
+      if (renderer->context)
+        {
+          gboolean follow_theme = FALSE;
+
+          g_object_get (renderer->context->gimp->config,
+                        "viewables-follow-theme", &follow_theme,
+                        NULL);
+          if (follow_theme)
+            {
+              GtkStyleContext *style;
+              GdkRGBA         *fg_color = NULL;
+              GdkRGBA         *bg_color = NULL;
+
+              style = gtk_widget_get_style_context (widget);
+              gtk_style_context_get (style, gtk_style_context_get_state (style),
+                                     GTK_STYLE_PROPERTY_COLOR,            &fg_color,
+                                     GTK_STYLE_PROPERTY_BACKGROUND_COLOR, &bg_color,
+                                     NULL);
+              if (fg_color && bg_color)
+                {
+                  color = gegl_color_new (NULL);
+                  gegl_color_set_rgba_with_space (color,
+                                                  fg_color->red, fg_color->green, fg_color->blue, 1.0,
+                                                  NULL);
+
+                  background = gegl_color_new (NULL);
+                  gegl_color_set_rgba_with_space (background,
+                                                  bg_color->red, bg_color->green, bg_color->blue, 1.0,
+                                                  NULL);
+                }
+              g_clear_pointer (&fg_color, gdk_rgba_free);
+              g_clear_pointer (&bg_color, gdk_rgba_free);
+            }
+        }
+
+      pixbuf = gimp_viewable_get_new_pixbuf (renderer->viewable,
+                                             renderer->context,
+                                             width, height, color, background);
+
+      g_clear_object (&color);
+      g_clear_object (&background);
+
+      return pixbuf;
+    }
 
   return NULL;
 }
 
 static gboolean
-gimp_container_icon_view_get_1_selected (GimpContainerIconView *icon_view,
-                                         GtkTreeIter           *iter)
+gimp_container_icon_view_get_selected_single (GimpContainerIconView  *icon_view,
+                                              GtkTreeIter            *iter)
 {
   GList    *selected_items;
   gboolean  retval;
@@ -847,7 +875,7 @@ gimp_container_icon_view_get_1_selected (GimpContainerIconView *icon_view,
   if (g_list_length (selected_items) == 1)
     {
       gtk_tree_model_get_iter (GTK_TREE_MODEL (icon_view->model), iter,
-                               selected_items->data);
+                               (GtkTreePath *) selected_items->data);
 
       retval = TRUE;
     }
@@ -861,6 +889,106 @@ gimp_container_icon_view_get_1_selected (GimpContainerIconView *icon_view,
   return retval;
 }
 
+static gint
+gimp_container_icon_view_get_selected (GimpContainerView    *view,
+                                       GList               **items,
+                                       GList               **paths)
+{
+  GimpContainerIconView *icon_view = GIMP_CONTAINER_ICON_VIEW (view);
+  GList                 *selected_paths;
+  gint                   selected_count;
+  GimpContainer         *container;
+
+  container = gimp_container_view_get_container (view);
+
+  if (container)
+    {
+      const gchar *signal_name;
+      GimpContext *context;
+      GType        children_type;
+
+      context       = gimp_container_view_get_context (view);
+      children_type = gimp_container_get_children_type (container);
+      signal_name   = gimp_context_type_to_signal_name (children_type);
+
+      /* As a special case, for containers tied to a context object, we
+       * look up this object as being selected.
+       * */
+      if (signal_name && context)
+        {
+          GimpObject  *object;
+
+          object  = gimp_context_get_by_type (context, children_type);
+
+          selected_count = object ? 1 : 0;
+          if (items)
+            {
+              if (object)
+                *items = g_list_prepend (NULL, object);
+              else
+                *items = NULL;
+            }
+          if (paths)
+            *paths = NULL;
+
+          return selected_count;
+        }
+    }
+
+  selected_paths = gtk_icon_view_get_selected_items (icon_view->view);
+  selected_count = g_list_length (selected_paths);
+
+  if (items)
+    {
+      GList *removed_paths = NULL;
+      GList *list;
+
+      *items = NULL;
+
+      for (list = selected_paths;
+           list;
+           list = g_list_next (list))
+        {
+          GtkTreeIter       iter;
+          GimpViewRenderer *renderer;
+
+          gtk_tree_model_get_iter (GTK_TREE_MODEL (icon_view->model), &iter,
+                                   (GtkTreePath *) list->data);
+
+          gtk_tree_model_get (icon_view->model, &iter,
+                              GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
+                              -1);
+
+          if (renderer->viewable)
+            *items = g_list_prepend (*items, renderer->viewable);
+          else
+            /* Remove from the selected_paths list but at the end, in order not
+             * to break the for loop.
+             */
+            removed_paths = g_list_prepend (removed_paths, list);
+
+          g_object_unref (renderer);
+        }
+      *items = g_list_reverse (*items);
+
+      for (list = removed_paths; list; list = list->next)
+        {
+          GList *remove_list = list->data;
+
+          selected_paths = g_list_remove_link (selected_paths, remove_list);
+          gtk_tree_path_free (remove_list->data);
+        }
+      g_list_free_full (removed_paths, (GDestroyNotify) g_list_free);
+    }
+
+  if (paths)
+    *paths = selected_paths;
+  else
+    g_list_free_full (selected_paths, (GDestroyNotify) gtk_tree_path_free);
+
+  return selected_count;
+}
+
 static void
 gimp_container_icon_view_trigger_redraw (GimpContainerIconView *view)
 {
