@@ -528,7 +528,7 @@ gimp_data_factory_view_get_data_factory (GimpDataFactoryView *factory_view)
 }
 
 GType
-gimp_data_factory_view_get_child_type (GimpDataFactoryView *factory_view)
+gimp_data_factory_view_get_children_type (GimpDataFactoryView *factory_view)
 {
   g_return_val_if_fail (GIMP_IS_DATA_FACTORY_VIEW (factory_view), G_TYPE_NONE);
 
@@ -567,7 +567,7 @@ gimp_data_factory_view_select_item (GimpContainerEditor *editor,
       GimpContainerView *container_view = GIMP_CONTAINER_VIEW (editor->view);
       GList             *active_items   = NULL;
 
-      gimp_container_view_get_selected (container_view, &active_items);
+      gimp_container_view_get_selected (container_view, &active_items, NULL);
       gimp_tag_entry_set_selected_items (GIMP_TAG_ENTRY (view->priv->assign_tag_entry),
                                          active_items);
       g_list_free (active_items);
