@@ -47,10 +47,10 @@
 
 #include "gimpcontainertreestore.h"
 #include "gimpcontainerview.h"
-#include "gimpcontainerview-cruft.h"
 #include "gimpdnd.h"
 #include "gimpdrawabletreeview.h"
 #include "gimpdrawabletreeview-filters.h"
+#include "gimplayertreeview.h" /* FIXME */
 #include "gimpviewrenderer.h"
 
 #include "gimp-intl.h"
@@ -77,14 +77,15 @@ static void     gimp_drawable_tree_view_style_updated (GtkWidget
 
 static void     gimp_drawable_tree_view_set_container (GimpContainerView          *view,
                                                        GimpContainer              *container);
-static gboolean gimp_drawable_tree_view_set_selected  (GimpContainerView          *view,
-                                                       GList                      *items);
-
 static gpointer gimp_drawable_tree_view_insert_item   (GimpContainerView          *view,
                                                        GimpViewable               *viewable,
                                                        gpointer                    parent_insert_data,
                                                        gint                        index);
 
+static gboolean gimp_drawable_tree_view_select_items  (GimpContainerView          *view,
+                                                       GList                      *items,
+                                                       GList                      *paths);
+
 static gboolean gimp_drawable_tree_view_drop_possible (GimpContainerTreeView      *view,
                                                        GimpDndType                 src_type,
                                                        GList                      *src_viewables,
@@ -182,9 +183,8 @@ gimp_drawable_tree_view_view_iface_init (GimpContainerViewInterface *iface)
   parent_view_iface = g_type_interface_peek_parent (iface);
 
   iface->set_container = gimp_drawable_tree_view_set_container;
-  iface->set_selected  = gimp_drawable_tree_view_set_selected;
-
   iface->insert_item   = gimp_drawable_tree_view_insert_item;
+  iface->select_items  = gimp_drawable_tree_view_select_items;
 }
 
 static void
@@ -238,8 +238,6 @@ gimp_drawable_tree_view_constructed (GObject *object)
   gtk_tree_view_column_set_attributes (column, view->priv->filters_cell,
                                        "active",
                                        view->priv->model_column_filters,
-                                       "visible",
-                                       view->priv->model_column_filters,
                                        NULL);
 
   gimp_container_tree_view_add_toggle_cell (tree_view,
@@ -292,10 +290,6 @@ gimp_drawable_tree_view_style_updated (GtkWidget *widget)
   GtkIconSize           old_size;
   GtkIconSize           button_icon_size = GTK_ICON_SIZE_SMALL_TOOLBAR;
 
-  gtk_widget_style_get (widget,
-                        "button-icon-size", &button_icon_size,
-                        NULL);
-
   gtk_icon_size_lookup (button_icon_size, &pixel_icon_size, NULL);
 
   gtk_image_get_icon_name (GTK_IMAGE (view->priv->filters_header_image),
@@ -343,9 +337,36 @@ gimp_drawable_tree_view_set_container (GimpContainerView *view,
                                  view);
 }
 
+static gpointer
+gimp_drawable_tree_view_insert_item (GimpContainerView *view,
+                                     GimpViewable      *viewable,
+                                     gpointer           parent_insert_data,
+                                     gint               index)
+{
+  GimpContainerTreeView *tree_view     = GIMP_CONTAINER_TREE_VIEW (view);
+  GimpDrawableTreeView  *drawable_view = GIMP_DRAWABLE_TREE_VIEW (view);
+  GimpContainer         *filters;
+  GtkTreeIter           *iter;
+  gint                   n_filters;
+
+  iter = parent_view_iface->insert_item (view, viewable,
+                                         parent_insert_data, index);
+
+  filters   = gimp_drawable_get_filters (GIMP_DRAWABLE (viewable));
+  n_filters = gimp_container_get_n_children (filters);
+
+  gtk_tree_store_set (GTK_TREE_STORE (tree_view->model), iter,
+                      drawable_view->priv->model_column_filters,
+                      n_filters > 0,
+                      -1);
+
+  return iter;
+}
+
 static gboolean
-gimp_drawable_tree_view_set_selected (GimpContainerView *view,
-                                      GList             *items)
+gimp_drawable_tree_view_select_items (GimpContainerView *view,
+                                      GList             *items,
+                                      GList             *paths)
 {
   GimpItemTreeView *item_view = GIMP_ITEM_TREE_VIEW (view);
   GimpImage        *image     = gimp_item_tree_view_get_image (item_view);
@@ -373,38 +394,11 @@ gimp_drawable_tree_view_set_selected (GimpContainerView *view,
     }
 
   if (success)
-    success = parent_view_iface->set_selected (view, items);
+    success = parent_view_iface->select_items (view, items, paths);
 
   return success;
 }
 
-static gpointer
-gimp_drawable_tree_view_insert_item (GimpContainerView *view,
-                                     GimpViewable      *viewable,
-                                     gpointer           parent_insert_data,
-                                     gint               index)
-{
-  GimpContainerTreeView *tree_view     = GIMP_CONTAINER_TREE_VIEW (view);
-  GimpDrawableTreeView  *drawable_view = GIMP_DRAWABLE_TREE_VIEW (view);
-  GimpContainer         *filters;
-  GtkTreeIter           *iter;
-  gint                   n_filters;
-
-  iter = parent_view_iface->insert_item (view, viewable,
-                                         parent_insert_data, index);
-
-  filters   = gimp_drawable_get_filters (GIMP_DRAWABLE (viewable));
-  n_filters = gimp_container_get_n_children (filters);
-
-  gtk_tree_store_set (GTK_TREE_STORE (tree_view->model), iter,
-                      drawable_view->priv->model_column_filters,
-                      n_filters > 0,
-                      -1);
-
-  return iter;
-}
-
-
 /*  GimpContainerTreeView methods  */
 
 static gboolean
@@ -515,7 +509,7 @@ static void
 gimp_drawable_tree_view_set_image (GimpItemTreeView *view,
                                    GimpImage        *image)
 {
-  GimpDrawableTreeView *drawable_view = GIMP_DRAWABLE_TREE_VIEW (view);
+  GimpDrawableTreeView  *drawable_view = GIMP_DRAWABLE_TREE_VIEW (view);
 
   if (gimp_item_tree_view_get_image (view))
     {
@@ -524,10 +518,6 @@ gimp_drawable_tree_view_set_image (GimpItemTreeView *view,
       g_signal_handlers_disconnect_by_func (gimp_item_tree_view_get_image (view),
                                             gimp_drawable_tree_view_floating_selection_changed,
                                             view);
-      g_signal_handlers_disconnect_by_func (gimp_item_tree_view_get_image (view)->gimp->config,
-                                            G_CALLBACK (gimp_drawable_tree_view_style_updated),
-                                            view);
-
     }
 
   GIMP_ITEM_TREE_VIEW_CLASS (parent_class)->set_image (view, image);
@@ -538,19 +528,6 @@ gimp_drawable_tree_view_set_image (GimpItemTreeView *view,
                         "floating-selection-changed",
                         G_CALLBACK (gimp_drawable_tree_view_floating_selection_changed),
                         view);
-
-      g_signal_connect_object (image->gimp->config,
-                               "notify::theme",
-                               G_CALLBACK (gimp_drawable_tree_view_style_updated),
-                               view, G_CONNECT_AFTER | G_CONNECT_SWAPPED);
-      g_signal_connect_object (image->gimp->config,
-                               "notify::override-theme-icon-size",
-                               G_CALLBACK (gimp_drawable_tree_view_style_updated),
-                               view, G_CONNECT_AFTER | G_CONNECT_SWAPPED);
-      g_signal_connect_object (image->gimp->config,
-                               "notify::custom-icon-size",
-                               G_CALLBACK (gimp_drawable_tree_view_style_updated),
-                               view, G_CONNECT_AFTER | G_CONNECT_SWAPPED);
     }
 }
 
@@ -561,16 +538,13 @@ static void
 gimp_drawable_tree_view_floating_selection_changed (GimpImage            *image,
                                                     GimpDrawableTreeView *view)
 {
-  GType  item_type;
   GList *items;
 
-  item_type = GIMP_ITEM_TREE_VIEW_GET_CLASS (view)->item_type;
-
-  items = gimp_image_get_selected_items (image, item_type);
+  items = GIMP_ITEM_TREE_VIEW_GET_CLASS (view)->get_selected_items (image);
   items = g_list_copy (items);
 
   /*  update button states  */
-  gimp_container_view_set_selected (GIMP_CONTAINER_VIEW (view), items);
+  gimp_container_view_select_items (GIMP_CONTAINER_VIEW (view), items);
 
   g_list_free (items);
 }
@@ -585,8 +559,8 @@ gimp_drawable_tree_view_filters_changed (GimpDrawable         *drawable,
   GimpContainerTreeView *tree_view = GIMP_CONTAINER_TREE_VIEW (view);
   GtkTreeIter           *iter;
 
-  iter = _gimp_container_view_lookup (GIMP_CONTAINER_VIEW (view),
-                                      (GimpViewable *) drawable);
+  iter = gimp_container_view_lookup (GIMP_CONTAINER_VIEW (view),
+                                     (GimpViewable *) drawable);
 
   if (iter)
     {
