@@ -25,16 +25,33 @@
 #include "gimpcontainertreeview.h"
 
 
-typedef GimpItem * (* GimpNewItemFunc)  (GimpImage *image);
+typedef GimpContainer * (* GimpGetContainerFunc) (GimpImage *image);
+typedef GimpItem      * (* GimpGetItemFunc)      (GimpImage *image);
+typedef void            (* GimpSetItemFunc)      (GimpImage *image,
+                                                  GimpItem  *item);
+typedef GList         * (* GimpGetItemsFunc)     (GimpImage *image);
+typedef void            (* GimpSetItemsFunc)     (GimpImage *image,
+                                                  GList     *items);
+typedef void            (* GimpAddItemFunc)      (GimpImage *image,
+                                                  GimpItem  *item,
+                                                  GimpItem  *parent,
+                                                  gint       index,
+                                                  gboolean   push_undo);
+typedef void            (* GimpRemoveItemFunc)   (GimpImage *image,
+                                                  GimpItem  *item,
+                                                  gboolean   push_undo,
+                                                  GimpItem  *new_active);
+typedef GimpItem      * (* GimpNewItemFunc)      (GimpImage *image);
 
-typedef gboolean   (* GimpIsLockedFunc) (GimpItem  *item);
-typedef gboolean   (* GimpCanLockFunc)  (GimpItem  *item);
-typedef void       (* GimpSetLockFunc)  (GimpItem  *item,
-                                         gboolean   lock,
-                                         gboolean   push_undo);
-typedef GimpUndo * (*GimpUndoLockPush)  (GimpImage     *image,
-                                         const gchar   *undo_desc,
-                                         GimpItem      *item);
+
+typedef gboolean        (* GimpIsLockedFunc)     (GimpItem  *item);
+typedef gboolean        (* GimpCanLockFunc)      (GimpItem  *item);
+typedef void            (* GimpSetLockFunc)      (GimpItem  *item,
+                                                  gboolean   lock,
+                                                  gboolean   push_undo);
+typedef GimpUndo      * (*GimpUndoLockPush)      (GimpImage     *image,
+                                                  const gchar   *undo_desc,
+                                                  GimpItem      *item);
 
 
 #define GIMP_TYPE_ITEM_TREE_VIEW            (gimp_item_tree_view_get_type ())
@@ -47,14 +64,12 @@ typedef GimpUndo * (*GimpUndoLockPush)  (GimpImage     *image,
 
 typedef struct _GimpItemTreeViewClass   GimpItemTreeViewClass;
 typedef struct _GimpItemTreeViewPrivate GimpItemTreeViewPrivate;
-typedef struct _GimpItemTreeViewSearch  GimpItemTreeViewSearch;
 
 struct _GimpItemTreeView
 {
   GimpContainerTreeView    parent_instance;
 
   GimpItemTreeViewPrivate *priv;
-  GimpItemTreeViewSearch  *search;
 };
 
 struct _GimpItemTreeViewClass
@@ -69,6 +84,11 @@ struct _GimpItemTreeViewClass
   const gchar          *signal_name;
 
   /*  virtual functions for manipulating the image's item tree  */
+  GimpGetContainerFunc  get_container;
+  GimpGetItemsFunc      get_selected_items;
+  GimpSetItemsFunc      set_selected_items;
+  GimpAddItemFunc       add_item;
+  GimpRemoveItemFunc    remove_item;
   GimpNewItemFunc       new_item;
 
   /*  action names  */
