@@ -119,6 +119,8 @@ gimp_sample_point_editor_class_init (GimpSamplePointEditorClass *klass)
                                                          TRUE,
                                                          GIMP_PARAM_READWRITE |
                                                          G_PARAM_CONSTRUCT));
+
+  gtk_widget_class_set_css_name (widget_class, "GimpSamplePointEditor");
 }
 
 static void
@@ -553,7 +555,7 @@ gimp_sample_point_editor_update (GimpSamplePointEditor *editor)
           GimpSamplePoint   *sample_point = list->data;
           const Babl        *format;
           gdouble            pixel[4];
-          GimpRGB            color;
+          GeglColor         *color;
           GimpColorPickMode  pick_mode;
           gint               x;
           gint               y;
@@ -563,6 +565,7 @@ gimp_sample_point_editor_update (GimpSamplePointEditor *editor)
 
           gimp_sample_point_get_position (sample_point, &x, &y);
 
+          color = gegl_color_new ("black");
           if (gimp_image_pick_color (image_editor->image, NULL,
                                      x, y,
                                      FALSE,
@@ -572,9 +575,7 @@ gimp_sample_point_editor_update (GimpSamplePointEditor *editor)
                                      pixel,
                                      &color))
             {
-              gimp_color_frame_set_color (color_frame, FALSE,
-                                          format, pixel, &color,
-                                          x, y);
+              gimp_color_frame_set_color (color_frame, FALSE, color, x, y);
             }
           else
             {
@@ -584,6 +585,8 @@ gimp_sample_point_editor_update (GimpSamplePointEditor *editor)
           pick_mode = gimp_sample_point_get_pick_mode (sample_point);
 
           gimp_color_frame_set_mode (color_frame, pick_mode);
+
+          g_object_unref (color);
         }
     }
 
