@@ -114,6 +114,8 @@ static cairo_pattern_t *
                  gimp_view_renderer_create_background (GimpViewRenderer   *renderer,
                                                        GtkWidget          *widget);
 
+static void      gimp_view_renderer_redraw            (GimpViewRenderer   *renderer);
+
 
 G_DEFINE_TYPE_WITH_PRIVATE (GimpViewRenderer, gimp_view_renderer, G_TYPE_OBJECT)
 
@@ -285,8 +287,26 @@ gimp_view_renderer_set_context (GimpViewRenderer *renderer,
 
   if (context != renderer->context)
     {
-      GIMP_VIEW_RENDERER_GET_CLASS (renderer)->set_context (renderer,
-                                                            context);
+      if (renderer->context)
+        {
+          g_signal_handlers_disconnect_by_func (renderer->context->gimp->config,
+                                                G_CALLBACK (gimp_view_renderer_redraw),
+                                                renderer);
+        }
+
+      GIMP_VIEW_RENDERER_GET_CLASS (renderer)->set_context (renderer, context);
+
+      if (renderer->context)
+        {
+          g_signal_connect_object (renderer->context->gimp->config,
+                                   "notify::viewables-follow-theme",
+                                   G_CALLBACK (gimp_view_renderer_redraw),
+                                   renderer, G_CONNECT_SWAPPED);
+          g_signal_connect_object (renderer->context->gimp->config,
+                                   "notify::theme-color-scheme",
+                                   G_CALLBACK (gimp_view_renderer_redraw),
+                                   renderer, G_CONNECT_SWAPPED);
+        }
 
       if (renderer->viewable)
         gimp_view_renderer_invalidate (renderer);
@@ -804,22 +824,63 @@ gimp_view_renderer_real_render (GimpViewRenderer *renderer,
   GdkPixbuf   *pixbuf;
   GimpTempBuf *temp_buf;
   const gchar *icon_name;
+  GeglColor   *color        = NULL;
+  GeglColor   *background   = NULL;
   gint         scale_factor = gtk_widget_get_scale_factor (widget);
 
+  if (renderer->context)
+    {
+      gboolean follow_theme = FALSE;
+
+      g_object_get (renderer->context->gimp->config,
+                    "viewables-follow-theme", &follow_theme,
+                    NULL);
+      if (follow_theme)
+        {
+          GtkStyleContext *style;
+          GdkRGBA         *fg_color = NULL;
+          GdkRGBA         *bg_color = NULL;
+
+          style = gtk_widget_get_style_context (widget);
+          gtk_style_context_get (style, gtk_style_context_get_state (style),
+                                 GTK_STYLE_PROPERTY_COLOR,            &fg_color,
+                                 GTK_STYLE_PROPERTY_BACKGROUND_COLOR, &bg_color,
+                                 NULL);
+          if (fg_color && bg_color)
+            {
+              color = gegl_color_new (NULL);
+              gegl_color_set_rgba_with_space (color,
+                                              fg_color->red, fg_color->green, fg_color->blue, 1.0,
+                                              NULL);
+
+              background = gegl_color_new (NULL);
+              gegl_color_set_rgba_with_space (background,
+                                              bg_color->red, bg_color->green, bg_color->blue, 1.0,
+                                              NULL);
+            }
+          g_clear_pointer (&fg_color, gdk_rgba_free);
+          g_clear_pointer (&bg_color, gdk_rgba_free);
+        }
+    }
+
   pixbuf = gimp_viewable_get_pixbuf (renderer->viewable,
                                      renderer->context,
                                      renderer->width  * scale_factor,
-                                     renderer->height * scale_factor);
+                                     renderer->height * scale_factor,
+                                     color, background);
   if (pixbuf)
     {
       gimp_view_renderer_render_pixbuf (renderer, widget, pixbuf);
+      g_clear_object (&color);
+      g_clear_object (&background);
       return;
     }
 
   temp_buf = gimp_viewable_get_preview (renderer->viewable,
                                         renderer->context,
                                         renderer->width,
-                                        renderer->height);
+                                        renderer->height,
+                                        color, background);
   if (temp_buf)
     {
       gimp_view_renderer_render_temp_buf_simple (renderer, widget, temp_buf);
@@ -828,6 +889,9 @@ gimp_view_renderer_real_render (GimpViewRenderer *renderer,
 
   icon_name = gimp_viewable_get_icon_name (renderer->viewable);
   gimp_view_renderer_render_icon (renderer, widget, icon_name);
+
+  g_clear_object (&color);
+  g_clear_object (&background);
 }
 
 static void
@@ -1176,6 +1240,45 @@ gimp_view_render_temp_buf_to_surface (GimpViewRenderer *renderer,
     case GIMP_VIEW_BG_WHITE:
       cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
       break;
+
+    case GIMP_VIEW_BG_USE_STYLE:
+      {
+        GtkStyleContext *style;
+        GdkRGBA         *color = NULL;
+
+        /* Try to get the parent's color first to prevent issues with
+         * changes in preselected backgrounds. If not set though, go for
+         * the current's widget's color instead */
+        style = gtk_widget_get_style_context (gtk_widget_get_parent (widget));
+        gtk_style_context_get (style, gtk_style_context_get_state (style),
+                               GTK_STYLE_PROPERTY_BACKGROUND_COLOR, &color,
+                               NULL);
+
+        if (color             &&
+            color->red   == 0 &&
+            color->green == 0 &&
+            color->blue  == 0)
+          {
+            gdk_rgba_free (color);
+
+            style = gtk_widget_get_style_context (widget);
+            gtk_style_context_get (style, gtk_style_context_get_state (style),
+                                   GTK_STYLE_PROPERTY_BACKGROUND_COLOR, &color,
+                                   NULL);
+          }
+
+        if (color)
+          {
+            cairo_set_source_rgb (cr, color->red, color->green, color->blue);
+
+            gdk_rgba_free (color);
+          }
+        else
+          {
+            cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
+          }
+      }
+      break;
     }
 
   cairo_paint (cr);
@@ -1205,6 +1308,10 @@ gimp_view_render_temp_buf_to_surface (GimpViewRenderer *renderer,
         case GIMP_VIEW_BG_WHITE:
           cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
           break;
+
+        case GIMP_VIEW_BG_USE_STYLE:
+        default:
+          break;
         }
 
       cairo_fill (cr);
@@ -1327,31 +1434,26 @@ gimp_view_render_temp_buf_to_surface (GimpViewRenderer *renderer,
       dest += y * dest_stride + x * 4;
 
       fish = babl_fish (temp_buf_format,
-                        babl_format ("cairo-RGB24"));
+                        babl_format ("R~G~B~ u8"));
 
-      for (i = y; i < (y + height); i++)
+      for (i = 0; i < height; i++)
         {
-          const guchar *s = src;
-          guchar       *d = dest;
+          guchar        line[width * 3];
+          const guchar *s = line;
+          guint32      *d = (guint32*) dest;
           gint          j;
 
-          for (j = x; j < (x + width); j++, d += 4, s += bytes)
+          babl_process (fish, src, line, width);
+
+          for (j = 0; j < width; j++)
             {
-              if (bytes > 2)
-                {
-                  guchar pixel[4] = { s[channel], s[channel], s[channel], 255 };
+              *d = s[channel] | s[channel] << 8 | s[channel] << 16;
 
-                  babl_process (fish, pixel, d, 1);
-                }
-              else
-                {
-                  guchar pixel[2] = { s[channel], 255 };
-
-                  babl_process (fish, pixel, d, 1);
-                }
+              s += 3;
+              d += 1;
             }
 
-          src += rowstride;
+          src  += rowstride;
           dest += dest_stride;
         }
 
@@ -1389,3 +1491,10 @@ gimp_view_renderer_create_background (GimpViewRenderer *renderer,
 
   return pattern;
 }
+
+static void
+gimp_view_renderer_redraw (GimpViewRenderer *renderer)
+{
+  gimp_view_renderer_invalidate (renderer);
+  gimp_view_renderer_update (renderer);
+}
