@@ -20,10 +20,13 @@
 
 #include "config.h"
 
+#include <string.h>
+
 #include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpcolor/gimpcolor.h"
+#include "libgimpmath/gimpmath.h"
 #include "libgimpwidgets/gimpwidgets.h"
 
 #include "widgets-types.h"
@@ -54,12 +57,9 @@ gimp_view_renderer_palette_class_init (GimpViewRendererPaletteClass *klass)
   GObjectClass          *object_class   = G_OBJECT_CLASS (klass);
   GimpViewRendererClass *renderer_class = GIMP_VIEW_RENDERER_CLASS (klass);
 
-  object_class->finalize          = gimp_view_renderer_palette_finalize;
+  object_class->finalize = gimp_view_renderer_palette_finalize;
 
-  renderer_class->default_bg      = GIMP_VIEW_BG_STYLE;
-  renderer_class->follow_theme_bg = GIMP_VIEW_BG_STYLE;
-
-  renderer_class->render          = gimp_view_renderer_palette_render;
+  renderer_class->render = gimp_view_renderer_palette_render;
 }
 
 static void
@@ -83,6 +83,9 @@ gimp_view_renderer_palette_render (GimpViewRenderer *renderer,
   GimpViewRendererPalette *renderpal = GIMP_VIEW_RENDERER_PALETTE (renderer);
   GimpPalette             *palette;
   GimpColorTransform      *transform;
+  GtkStyleContext         *style;
+  GdkRGBA                 *bg_color = NULL;
+  guchar                   bg_rgb[3];
   guchar                  *row;
   guchar                  *dest;
   GList                   *list;
@@ -96,6 +99,25 @@ gimp_view_renderer_palette_render (GimpViewRenderer *renderer,
   if (gimp_palette_get_n_colors (palette) == 0)
     return;
 
+  /* Get background colors from context */
+  style = gtk_widget_get_style_context (widget);
+  gtk_style_context_get (style, gtk_style_context_get_state (style),
+                         GTK_STYLE_PROPERTY_BACKGROUND_COLOR, &bg_color,
+                         NULL);
+
+  if (bg_color)
+    {
+      bg_rgb[0] = (gint) (bg_color->red * 255);
+      bg_rgb[1] = (gint) (bg_color->green * 255);
+      bg_rgb[2] = (gint) (bg_color->blue * 255);
+
+      gdk_rgba_free (bg_color);
+    }
+  else
+    {
+      bg_rgb[0] = bg_rgb[1] = bg_rgb[2] = 255;
+    }
+
   grid_width = renderpal->draw_grid ? 1 : 0;
 
   if (renderpal->cell_size > 0)
@@ -140,7 +162,7 @@ gimp_view_renderer_palette_render (GimpViewRenderer *renderer,
   list = gimp_palette_get_colors (palette);
 
   if (! renderer->surface)
-    renderer->surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
+    renderer->surface = cairo_image_surface_create (CAIRO_FORMAT_RGB24,
                                                     renderer->width,
                                                     renderer->height);
 
@@ -161,7 +183,7 @@ gimp_view_renderer_palette_render (GimpViewRenderer *renderer,
 
       if ((y % renderpal->cell_height) == 0)
         {
-          guchar  rgba[4];
+          guchar  rgb[3];
           gint    n = 0;
           guchar *d = row;
 
@@ -178,45 +200,45 @@ gimp_view_renderer_palette_render (GimpViewRenderer *renderer,
                       n++;
 
                       /* TODO: render directly to widget color space! */
-                      gegl_color_get_pixel (entry->color,
-                                            babl_format ("R'G'B'A u8"), rgba);
+                      gegl_color_get_pixel (entry->color, babl_format ("R'G'B' u8"), rgb);
                     }
                   else
                     {
-                      gint i;
-
                       /* Background color (not entries, not grids) */
-                      for (i = 0; i < 4; i++)
-                        rgba[i] = 0;
+                      for (gint i = 0; i < 3; i++)
+                        rgb[i] = bg_rgb[i];
                     }
                 }
 
               if (renderpal->draw_grid && (x % renderpal->cell_width) == 0)
                 {
                   /* Vertical grid lines */
-                  GIMP_CAIRO_ARGB32_SET_PIXEL (d, 0, 0, 0, 0);
+                  GIMP_CAIRO_RGB24_SET_PIXEL (d, bg_rgb[0], bg_rgb[1], bg_rgb[2]);
                 }
               else
                 {
                   /* Palette entry */
-                  GIMP_CAIRO_ARGB32_SET_PIXEL (d, rgba[0], rgba[1], rgba[2], rgba[3]);
+                  GIMP_CAIRO_RGB24_SET_PIXEL (d, rgb[0], rgb[1], rgb[2]);
                 }
             }
         }
 
       if (renderpal->draw_grid && (y % renderpal->cell_height) == 0)
         {
+          guchar *d = dest;
+
           /* Horizontal grid lines */
-          memset (dest, 0, renderer->width * 4);
+          for (x = 0; x < renderer->width; x++, d += 4)
+            GIMP_CAIRO_RGB24_SET_PIXEL (d, bg_rgb[0], bg_rgb[1], bg_rgb[2]);
         }
       else
         {
           if (transform)
             {
               gimp_color_transform_process_pixels (transform,
-                                                   babl_format ("cairo-ARGB32"),
+                                                   babl_format ("cairo-RGB24"),
                                                    row,
-                                                   babl_format ("cairo-ARGB32"),
+                                                   babl_format ("cairo-RGB24"),
                                                    dest,
                                                    renderer->width);
             }
