@@ -866,36 +866,36 @@ gimp_get_monitor_resolution (GdkMonitor *monitor,
   *yres = ROUND (y);
 }
 
-GeglColor *
+gboolean
 gimp_get_style_color (GtkWidget   *widget,
-                      const gchar *color_name)
+                      const gchar *property_name,
+                      GdkRGBA     *color)
 {
-  GtkStyleContext *style;
-  GdkRGBA         *gdk_rgba = NULL;
-  GeglColor       *color    = NULL;
+  GdkRGBA *c = NULL;
 
-  g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
-  g_return_val_if_fail (color_name != NULL, NULL);
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
+  g_return_val_if_fail (property_name != NULL, FALSE);
+  g_return_val_if_fail (color != NULL, FALSE);
 
-  style = gtk_widget_get_style_context (widget);
-  gtk_style_context_get (style, gtk_style_context_get_state (style),
-                         color_name, &gdk_rgba,
-                         NULL);
+  gtk_widget_style_get (widget,
+                        property_name, &c,
+                        NULL);
 
-  if (gdk_rgba)
+  if (c)
     {
-      color = gegl_color_new (NULL);
-      gegl_color_set_rgba_with_space (color,
-                                      gdk_rgba->red,
-                                      gdk_rgba->green,
-                                      gdk_rgba->blue,
-                                      1.0,
-                                      NULL);
+      *color = *c;
+      gdk_rgba_free (c);
 
-      gdk_rgba_free (gdk_rgba);
+      return TRUE;
     }
 
-  return color;
+  /* return ugly magenta to indicate that something is wrong */
+  color->red   = 1.0;
+  color->green = 1.0;
+  color->blue  = 0.0;
+  color->alpha = 1.0;
+
+  return FALSE;
 }
 
 void
@@ -922,8 +922,6 @@ gimp_window_set_hint (GtkWindow      *window,
 }
 
 /* similar to what we have in libgimp/gimpui.c */
-/* TODO: Restore when we use it on Windows */
-#ifndef G_OS_WIN32
 static GdkWindow *
 gimp_get_foreign_window (gpointer window)
 {
@@ -940,7 +938,6 @@ gimp_get_foreign_window (gpointer window)
 
   return NULL;
 }
-#endif
 
 void
 gimp_window_set_transient_for (GtkWindow    *window,
@@ -2595,9 +2592,7 @@ gimp_window_set_transient_cb (GtkWidget   *window,
                               GdkEventAny *event G_GNUC_UNUSED,
                               GBytes      *handle)
 {
-#ifndef G_OS_WIN32
   gboolean transient_set = FALSE;
-#endif
 
   g_return_if_fail (handle != NULL);
 
