@@ -1,12 +1,9 @@
-from gi.repository import Pango
 import os
 import sys
 
-image = Gimp.get_images()[0]
-
+image     = Gimp.get_images()[0]
 procedure = Gimp.get_pdb().lookup_procedure("file-png-export")
 config    = procedure.create_config()
-image.undo_disable()
 image.flatten()
 
 if image.get_width() > 1920 or image.get_height() > 1080:
@@ -22,4 +19,4 @@ config.set_property("time", False)
 config.set_property("format", "rgb8")
 retval = procedure.run(config)
 if retval.index(0) != Gimp.PDBStatusType.SUCCESS:
-  sys.exit(70)
+  sys.exit(os.EX_SOFTWARE)
